import exotethys
import os, pickle, shutil
import numpy as np
import pandas as pd
# import pylightcurve as plc
from pylightcurve_master import pylightcurve_self as plc
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
from scipy.interpolate import interp1d

from func_pylc import self_fp_over_fs


def update_limbfile(file_path, planet_object, filtername_list, filterchannelnum_list):
    """
    Repalce the string list int the 'sail.txt' file
    <<input>>
    file_path             : string                     ;
    planet_object         : pylightcurve planet object ; synthetic light curve data       ; generated from func:create_planet_tkdata
    filtername_list       : list(str)                  ; different filter name            ;
    filterchannelnum_list : list(int)                  ; the numbers of filter channel    ;
    
    <<output>>
    **                    : running                    ; execute the update code
    """
    
    file_data=""         # the written infromation for each line
    with open(file_path, "r", encoding="utf-8") as f:
        for line in f:
            # the following if-else func. is to replace the parameter which we what to use in exotethys files 
            if "output_path" in line:
                stellar_name = '-'.join(planet_object.name.split("-")[:-1])
                line = line.replace(line, f"output_path\t\t\t\t\tData_tk/limb/{stellar_name}\n")
            
            # stellar model will be selected by the diven model and its temperature.
            if "stellar_models_grid" in line:
                stellar_temperature = planet_object.stellar_temperature
                ldc_model           = planet_object.ldc_stellar_model
                if ldc_model == "atlas":
                    model_name = "!Phoenix_2018\t!Phoenix_2012_13\t!Phoenix_drift_2012\tAtlas_2000\t!Stagger_2015"
                    print("Exotethys will use model:Atlas_2000")
                    
                elif ldc_model == "phoenix":
                    if stellar_temperature<10000 and stellar_temperature>3000:
                        model_name = "!Phoenix_2018\tPhoenix_2012_13\t!Phoenix_drift_2012\t!Atlas_2000\t!Stagger_2015"
                        print("Exotethys will use model:Phoenix_2012_13")
                        
                    elif stellar_temperature<3000 and stellar_temperature>1500:
                        model_name = "!Phoenix_2018\t!Phoenix_2012_13\tPhoenix_drift_2012\t!Atlas_2000\t!Stagger_2015"
                        print("Exotethys will use model:Phoenix_drift_2012")
                        
                    else:
                        print("Please check the stellar temperature range!")
                        
                line = line.replace(line, f"stellar_models_grid\t\t\t{model_name}\n")
                
            # passband file need wavelength unit: A(10^-10m)
            if "passbands\t" in line:
                # add multiple filters
                passband_line = "passbands\t\t\t\t\t"
                for filtername in filtername_list:
                    passband_line = passband_line+filtername+".pass\t"
                passband_line = passband_line+"\n"
                line = line.replace(line,passband_line)
            if "wavelength_bins_files" in line:
                wavelength_bins_line = "wavelength_bins_files\t\t"
                index = 0
                for filterchannelnum in filterchannelnum_list:
                    # not need to bin
                    if filterchannelnum == 0:
                        wavelength_bins_line = wavelength_bins_line+"no_bins\t"
                        index = index+1
                        continue
                        
                    # need to pick up the filtername without "filter" inside the string
                    filter_pickname = '_'.join(filtername_list[index].split("_")[:-1])
                    wavelength_bins_line = wavelength_bins_line+filter_pickname+'_channel_'+str(filterchannelnum)+'.txt\t'
                    index = index+1
                    
                wavelength_bins_line = wavelength_bins_line+"\n"
                line = line.replace(line,wavelength_bins_line)
                    
            if "target_names" in line:
                line = line.replace(line,"target_names\t\t\t\t"+('-'.join(planet_object.name.split("-")[:-1]))+"\n")
            if "star_effective_temperature" in line:
                line = line.replace(line,"star_effective_temperature\t\t"+str(planet_object.stellar_temperature)+"\n")
            if "star_log_gravity" in line:
                line = line.replace(line,"star_log_gravity\t\t\t"+str(planet_object.stellar_logg)+"\n")
            if "star_metallicity" in line:
                if planet_object.ldc_stellar_model == 'phoenix':
                    line = line.replace(line,"star_metallicity\t\t\t0.\n")
                    print("PHOENIX models are only computed for solar metallicity stars. Setting stellar_metallicity = 0.")
                else:
                    line = line.replace(line,"star_metallicity\t\t\t"+str(planet_object.stellar_metallicity)+"\n")
            file_data += line

    with open(file_path,"w",encoding="utf-8") as f:
        f.write(file_data)
    
    return


# calculate the limb darkening coefficient by given .txt file
def run_exotethys(planet_object,
                  filtername_list, filterchannelnum_list,
                  re_exotethys=True, re_channel=True, verbose=True):
    """
    <<input>>
    planet_object         : pylightcurve planet object ; the inforamtion of planet parameter ; generated from func:create_planet_tkdata
    filtername_list       : list(string)               ; different filter name               ;
    filterchannelnum_list : list(int)                  ; the numbers of filter channel       ;
    re_exotethys          : boolen                     ; whether to re_exotethys or not
    re_channel            : boolen                     ; whether to re_channel or not
    verbose               : boolen                     ; print details or not
    
    <<progress>>
    **                    : running                    ; execute the ldc calculation code 
    
    <<output>>
    ldc['passbands']      : dict                       ; the limb darkening coefficents dict ; result calculated by Exotethys package
    """
    
    # check whether the filter need to be channeled or not
    filter_channel(filterchannelnum_list, filtername_list, re_channel, verbose)
    
    # because the planet name is HAT-P-xx-b  
    #                            HAT-P-xx-c
    #                            HAT-P-xx-...
    # the name will split into a list with "-"
    # need to merge list element
    stellar_name = '-'.join(planet_object.name.split("-")[:-1])
    path_save = 'Data_tk/limb/'+stellar_name
    
    # check if it already have file
    if not os.path.isfile(path_save+'/'+stellar_name+'_ldc.pickle'):
        print("Don't find the limb darkening coefficient file!!!")
        print("Start to call \"exotethys\" to calculate ")
        
        # check if there is the save folder
        if not os.path.isdir(path_save):
            os.makedirs(path_save)
        
        # update the file.txt exotethys need 
        shutil.copy("Data_tk/limb/sail_example.txt" , f"Data_tk/limb/{stellar_name}/sail.txt") 
        update_limbfile(f"Data_tk/limb/{stellar_name}/sail.txt", planet_object, filtername_list, filterchannelnum_list)

        # do the limb darkening coefficient calculation
        # (it will "interpolate the stellar profile" with different parameter)
        exotethys.sail.ldc_calculate(f"Data_tk/limb/{stellar_name}/sail.txt")
        
    # re_exotethys
    elif re_exotethys == True:
        print("Need to recalculate the limb darkening coefficient file!!!")
        print("Start to call \"exotethys\" to calculate ")
        
        # update the file.txt exotethys need
        shutil.copy("Data_tk/limb/sail_example.txt" , f"Data_tk/limb/{stellar_name}/sail.txt") 
        update_limbfile(f"Data_tk/limb/{stellar_name}/sail.txt",planet_object, filtername_list, filterchannelnum_list)
        
        # check if there is the save folder
        if not os.path.isdir(path_save):
            os.makedirs(path_save)

        # do the limb darkening coefficient calculation
        exotethys.sail.ldc_calculate(f"Data_tk/limb/{stellar_name}/sail.txt")  # it will "interpolate the stellar profile" with different parameter
    
    # load the result data and get coefficient
    print("Get the limb darkening coefficient file!!!")
    ldc = pickle.load(open(f"Data_tk/limb/{stellar_name}/{stellar_name}_ldc.pickle",'rb'),encoding='latin1')
    
    return ldc['passbands']



def filter_channel(filterchannelnum_list, filtername_list, re_channel, verbose):
    """
    <<input>>
    filterchannelnum_list : list(int) ; the numbers of filter channel
    filtername_list       : list(str) ; different filter name 
    re_channel            : boolen    ; whether to re_channel or not
    verbose               : boolen    ; print details or not
    
    <<output>>
    **                    : running   ; execute the update code
    """
    if not len(filterchannelnum_list)==len(filtername_list):
        print("filterchannelnum_list should have same len as filtername_list")
        return
    
    index = 0
    for filterchannelnum in filterchannelnum_list:
        
        filtername = filtername_list[index]    
        filter_pickname = '_'.join(filtername.split("_")[:-1])  # need to pick up the filtername without "filter" inside the string
        
        # if nobins, we do not need to generate the additional channel filter files
        if filterchannelnum == 0:
            print(f"The filter {filter_pickname} don't need to do filter channeling")
            index = index+1
            continue
            
        # if there was the default channel data, just pick the information and print them
        elif filterchannelnum == 'default' and not re_channel:
            print(f"The filter {filter_pickname} use the default filter channeling")
            
            df_filter  = pd.read_csv(f"Data_tk/passband/channel/{filter_pickname}_channel_default.txt",
                         sep='\t', header=None, names=['low', 'high'])    # load the filter data
            
            print(f"Divide {filter_pickname} into {len(df_filter)} wavelength channels")
            if verbose:
                print(df_filter)
                        
            fig = plt.figure(figsize=(8,6), dpi=100)
            channelplot = mpimg.imread(f'Data_tk/passband/channel/{filter_pickname}_channel_default'+'.png')
            plt.imshow(channelplot, aspect='auto')
            plt.axis('off')
            plt.show()
            
            index = index+1
            continue
            
        # if there was the channel data generated before, just pick the information and print them
        elif os.path.isfile(f"Data_tk/passband/channel/{filter_pickname}_channel_{filterchannelnum}.txt") and not re_channel:
            print(f"The {filter_pickname} filter channel file with {filterchannelnum} channel was generated before!")

            df_filter  = pd.read_csv(f"Data_tk/passband/channel/{filter_pickname}_channel_{filterchannelnum}.txt",
                                     sep='\t', header=None, names=['low', 'high'])    # load the filter data
            
            print(f"Divide {filter_pickname} into {filterchannelnum} wavelength channels")
            print(f"Each channel has a {(df_filter.loc[0]['high']-df_filter.loc[0]['low'])}(A) wavelength interval.")
            if verbose:
                print(df_filter)
            
            fig = plt.figure(figsize=(8,6), dpi=100)
            channelplot = mpimg.imread(f"Data_tk/passband/channel/{filter_pickname}_channel_{filterchannelnum}.png")
            plt.imshow(channelplot, aspect='auto')
            plt.axis('off')
            plt.show()
            
            index = index+1
            continue
            
        index     = index+1
        
        print(f"run/re_channel the filter channeling process of {filtername} in {filterchannelnum} number of channeling")
        
        if filterchannelnum == 'default':
            print(f"The filter {filter_pickname} use the default filter channeling")
            df_filter  = pd.read_csv(f"Data_tk/passband/{filtername}.pass", sep='\t',
                         header=None, names=['Wavelength(A)', 'tranmission rate'])    # load the filter data
            wavelength = df_filter['Wavelength(A)'].to_numpy()
            
            df  = pd.read_csv(f"Data_tk/passband/channel/{filter_pickname}_channel_default.txt",
                                sep='\t', header=None, names=['low', 'high'])    # load the filter data
            
            print(f"Divide {filter_pickname} into {len(df)} wavelength channels")
            print(df)
            
            # plot the channeling filter
            [df['low'].to_list()[0]-1]+df['high'].to_list()
            channel_point_df = np.array([df['low'].to_list()[0]-1]+df['high'].to_list())
            # beacuse the first interval will be left limit < interval <= right limit,
            # the first left limit will be missed, so we apprend one more point
            
            filterchannelnum_savefig = filterchannelnum
            filterchannelnum = len(df)
        
        else:
            df_filter  = pd.read_csv(f"Data_tk/passband/{filtername}.pass", sep='\t',
                                     header=None, names=['Wavelength(A)', 'tranmission rate'])    # load the filter data

            wavelength = df_filter['Wavelength(A)'].to_numpy()
            channel_point = np.linspace(wavelength.min(),wavelength.max(), filterchannelnum+1)

            df = pd.DataFrame()
            df['low']  = channel_point[:-1]
            df['high'] = channel_point[1:]

            df.to_csv(f"Data_tk/passband/channel/{filter_pickname}_channel_{filterchannelnum}.txt", sep='\t', index=False, header=False)

            print(f"Divide {filter_pickname} into {filterchannelnum} wavelength channels")
            print(f"Each channel has a {(channel_point[1]-channel_point[0])}(A) wavelength interval.")
            print(df)
        
            # plot the channeling filter
            channel_point_df = np.append(channel_point[0]-1, channel_point[1:])
            # beacuse the first interval will be left limit < interval <= right limit,
            # the first left limit will be missed, so we apprend one more point
            
            filterchannelnum_savefig = filterchannelnum
        
        # add the label of belonging channel for each row
        df_filter['channel'] = pd.cut(df_filter['Wavelength(A)'], channel_point_df, labels=list(range(filterchannelnum)))
        
        # plot
        plot_filterchannel(filterchannelnum, filter_pickname, df_filter, df, filterchannelnum_savefig)
        
    return


def plot_filterchannel(filterchannelnum, filter_pickname, df_filter, df, filterchannelnum_savefig):
    """
    <<input>>
    filterchannelnum         :  int        ; the numbers of filter channel
    filter_pickname          :  string     ; (original) filter name 
    df_filter                :  dataframe  ; the table of (original) filter transmission information in its range
    df                       :  dataframe  ; the table of (original) filter channel information with left/right edge
    filterchannelnum_savefig :  int/string ; the string when saving the channel filter profile figure
    
    <<output>>
    figure                   : figure       ;  under path: 'Data_tk/passband/channel/
    """

    colors = plt.cm.jet(np.linspace(0,1,filterchannelnum))  # the color map with gradually change (.jet is one kind of cmap)

    fig = plt.figure(figsize=(8,6), dpi=100, constrained_layout=True)
    fig.suptitle(f"The channeling {filter_pickname} filter with {filterchannelnum} channels")
    fig.patch.set_facecolor('white')
    ax  = fig.add_subplot(111)
    for i in range(filterchannelnum):
        ymean  = df_filter['tranmission rate'][df_filter.channel==i].mean()
        each_w = df_filter['Wavelength(A)'][df_filter.channel==i].to_list()
        each_t = df_filter['tranmission rate'][df_filter.channel==i].to_list()

        # consider the separated channel details (the upper and lower boundary probem when face separated line),
        # using interpolation to find the appropriate value when wavelength go to separate into two channels
        if i == 0:                      # when the left limit of boundary
            f_right = interp1d([each_w[-1], df_filter['Wavelength(A)'][df_filter.channel==i+1].to_list()[0]],
                               [each_t[-1], df_filter['tranmission rate'][df_filter.channel==i+1].to_list()[0]])

            each_w = [df.loc[i]['low']] + each_w + [df.loc[i]['high']         , df.loc[i]['high']]
            each_t = [0] + each_t + [f_right(df.loc[i]['high']), 0]

        elif i == filterchannelnum-1:   # when the right limit of boundary
            f_left  = interp1d([df_filter['Wavelength(A)'][df_filter.channel==i-1].to_list()[-1], each_w[0]],
                               [df_filter['tranmission rate'][df_filter.channel==i-1].to_list()[-1], each_t[0]])

            each_w = [df.loc[i]['low'], df.loc[i]['low']] + each_w + [df.loc[i]['high']]
            each_t = [0, f_left(df.loc[i]['low'])] + each_t + [0]

        else:                           # boundary of each channel
            f_left  = interp1d([df_filter['Wavelength(A)'][df_filter.channel==i-1].to_list()[-1], each_w[0]],
                               [df_filter['tranmission rate'][df_filter.channel==i-1].to_list()[-1], each_t[0]])
            f_right = interp1d([each_w[-1], df_filter['Wavelength(A)'][df_filter.channel==i+1].to_list()[0]],
                               [each_t[-1], df_filter['tranmission rate'][df_filter.channel==i+1].to_list()[0]])

            each_w = [df.loc[i]['low'], df.loc[i]['low']] + each_w + [df.loc[i]['high'], df.loc[i]['high']]
            each_t = [0, f_left(df.loc[i]['low'])] + each_t + [f_right(df.loc[i]['high']),0]

        each_w = np.array(each_w)
        each_t = np.array(each_t)

        ax.plot(each_w/1e4, each_t, color=colors[i])
        ax.annotate(f"c{i}", xy=(each_w.mean()/1e4, ymean))
        ax.set_xlabel('Wavelength(um)')
        ax.set_ylabel('tranmission rate')
        ax.set_ylim(0,1)

    plt.savefig(f"Data_tk/passband/channel/{filter_pickname}_channel_{filterchannelnum_savefig}.png")
    plt.show()
    
    return
        
    
def gen_filtername_allchannellist(ldc_filter, filtername_list, filterchannelnum_list, planet_tk):
    """
    <<input>>
    ldc_filter                : dict                       ; the limb darkening coefficents dict
    filtername_list           : list(str)                  ; different filter (original) name
    filterchannelnum_list     : list(int)                  ; the numbers of filter channel
    planet_tk                 : pylightcurve planet object ; the inforamtion of planet parameter ; generated from func:create_planet_tkdata

    <<output>>
    filtername_allchannellist :  list(str)  ;   different filter (+channel) name
    """
    
    filter_index = 0
    filtername_allchannellist = []
    for filter_name in list(ldc_filter.keys()):

        filter_pickname = filter_name.split(".pass")[0]
        filter_pickwave = '_'.join(filter_name.split(".pass")[1].split("_")[1:])

        if filtername_list[filter_index] != filter_pickname:
            filter_index= filter_index+1

        if (filterchannelnum_list[filter_index] == 0) and (filter_pickwave != ''):
            continue

        # here, the self_fp_over_fs need the filter name to load the "transmission rate"
        filter_fp= self_fp_over_fs(planet_tk.rp_over_rs,
                                     planet_tk.sma_over_rs,
                                     planet_tk.albedo,
                                     planet_tk.emissivity,
                                     planet_tk.stellar_temperature,
                                     filter_pickname,
                                     filter_pickwave)

        ldc1, ldc2, ldc3, ldc4 = ldc_filter[filter_name]['claret4']['coefficients']

        if filter_pickwave == '':
            print(filter_pickname, ldc_filter[filter_name]['claret4']['coefficients'])
            planet_tk.add_filter(filter_pickname+filter_pickwave,  planet_tk.rp_over_rs,
                                 ldc1, ldc2, ldc3, ldc4, filter_fp)

            filtername_allchannellist.append(filter_pickname)
        else:
            print(filter_pickname+'-'+filter_pickwave, ldc_filter[filter_name]['claret4']['coefficients'])
            planet_tk.add_filter(filter_pickname+'-'+filter_pickwave,  planet_tk.rp_over_rs,
                                 ldc1, ldc2, ldc3, ldc4, filter_fp)

            filtername_allchannellist.append(filter_pickname+'-'+filter_pickwave)
            
    return filtername_allchannellist


import json, os
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from astropy.time import Time


def plot_synlc_vsfilter(data, exposure_list,
                        mode, filtername_list,
                        filename,
                        ymax, ymin, save=True):
    """
    <<input>>
    data            : dictionary ; synthetic light curve data            ; generated from LC_syn_tk.py
    exposure_list   : list       ; different exposure time list 
    mode            : string     ; which type of mode will be plotted    ; consider "whole tranist time"/"observable time"
    filtername_list : list(str)  ; different filter name                 ;
    filename        : string     ; the figure saving path and name
    ymax            : float      ; the y-axis max
    ymin            : float      ; the y-axis min
    save            : boolen     ; whether save the figure or not        ; default= True
    
    <<output>>
    if mode=='obs':
        figure          : figure     ; under path: Result/{target_name}/light_curve/{target_date}/
    
    elif mode=='mod':
        figure          : figure     ; under path: Result/{target_name}/light_curve/
    """
    
    # check the filename and its path
    pathcheck_index  = filename.split('/').index("light_curve")
    pathcheck_length = len(filename.split('/')[pathcheck_index+1:])
    
    if mode=="mod" and pathcheck_length==1:
        title   = "model"
    elif mode=="obs" and pathcheck_length==2:
        title   = "observable"
    else:
        print("The filename has something wrong!")
        """
        if mode=='obs':
            figure          : figure     ; under path: Result/{target_name}/light_curve/{target_date}/
        elif mode=='mod':
            figure          : figure     ; under path: Result/{target_name}/light_curve/
        """
        return
    
    target_name  = data["planet_object"].name
    stellar_name = '-'.join(target_name.split('-')[:-1])
    target_date  = data["target_date"]
    
    # get the available transit date (by the stardrive datebase)  ; generated from Twinkle Stardrive tool(orbit)
    f = open(f"Data_tk/stardrive/{stellar_name}/{target_name}/transit-results.json")
    tk_avail_time = json.load(f)
    
    # the index of target_date in tk_avail_time
    df = pd.read_csv(f"Data_tk/stardrive/{stellar_name}/{target_name}/{target_name}_transit-date.csv")
    target_lc_num = df["Mid_date"][df["Mid_date"]==target_date].keys()[0]
        
    
    # plot !!!
    fs=28

    # generate the empty figure
    fig = plt.figure(figsize=(30,6), dpi=200, constrained_layout=True)
    if mode=="mod" and pathcheck_length==1:
        fig.suptitle(f"{target_name} transit light curve in ({title})",fontsize=fs)
    elif mode=="obs" and pathcheck_length==2:
        fig.suptitle(f"{target_name} on {target_date} transit in ({title})",fontsize=fs)
    fig.patch.set_facecolor('white')

    for i in range(len(filtername_list)):
        
        ax = fig.add_subplot(1, len(filtername_list), i+1)

        exp_label = str(exposure_list[i][0])
        
        # using "mode" parameter to determine which type
        # (mod=whole time / obs=consider earth blocked) of information is the giving data
        if mode=="mod":
            xdata   = data["time_array"][filtername_list[i]][f"exp_t{exp_label} [sec]"]
            ydata_l = data["tr_mod_array"][filtername_list[i]][f"exp_t{exp_label} [sec]"]
            ydata_s = data["mod_noise_array"][filtername_list[i]][f"exp_t{exp_label} [sec]"]

        elif mode=='obs':
            xdata_l = data['time_array'][filtername_list[i]][f"exp_t{exp_label} [sec]"]
            xdata_s = data['observable_array'][filtername_list[i]][f"exp_t{exp_label} [sec]"]
            ydata_l = data['tr_mod_array'][filtername_list[i]][f"exp_t{exp_label} [sec]"]
            ydata_s = data['obs_noise_array'][filtername_list[i]][f"exp_t{exp_label} [sec]"]
            
            
        
        ax.set_title(f"exp_time={exp_label}s filter:{filtername_list[i]}",fontsize=fs*0.8)
        
        if mode=="obs":           
            ax.plot((xdata_l-data["mid_time"])*24,
                     ydata_l,
                     label=filtername_list[i],
                     c='k', lw=4.0)
            ax.scatter((xdata_s-data["mid_time"])*24,
                        ydata_s,
                        label=f"exp_time:{exp_label}(s) / Photon noise:"+str(data["err_per_exp [ppm]"][filtername_list[i]][f"exp_t{exp_label} [sec]"])+"(ppm)",
                        c='C'+str(i), marker='o')
            
            # because the figure x axis is detemined as -time ~ mid_time=0 ~ +time
            # we should transform the orbit windows information in the same information type
            # (i.e. use the mid_time to be the base of each orbit window)
            t_start  = Time(tk_avail_time["transit"][target_lc_num]["start_time"])
            t_end    = Time(tk_avail_time["transit"][target_lc_num]["end_time"])

            t_mid    = Time((t_start.jd + t_end.jd)/2., format='jd')

            for ii in range(len(tk_avail_time["transit"][target_lc_num]["viewing_times"])):
                view_start  = Time(tk_avail_time["transit"][target_lc_num]["viewing_times"][ii][0])
                view_end    = Time(tk_avail_time["transit"][target_lc_num]["viewing_times"][ii][1])

                view_start  = (view_start.jd - t_mid.jd)*24
                view_end    = (view_end.jd   - t_mid.jd)*24

                if ii == len(tk_avail_time["transit"][target_lc_num]["viewing_times"])-1:
                    ax.fill_between([view_start,view_end],[0.1,0.1],[1.5,1.5],color='b',alpha=0.1,
                                     label="channel converage="+round(tk_avail_time["transit"][target_lc_num]["coverage"]*100,3)+"%")
                else:
                    ax.fill_between([view_start,view_end],[0.1,0.1],[1.5,1.5],color='b',alpha=0.1)
        
        elif mode=="mod":
            ax.plot((xdata-data["mid_time"])*24,
                     ydata_l,
                     label=filtername_list[i],
                     c='k', lw=4.0)
            ax.scatter((xdata-data["mid_time"])*24,
                        ydata_s,
                        label=f"exp_time:{exp_label}(s) / Photon noise:"+str(data["err_per_exp [ppm]"][filtername_list[i]][f"exp_t{exp_label} [sec]"])+"(ppm)",
                        c='C'+str(i), marker='o')

        ax.set_xlabel("Time Since Mid Transit [hours]",fontsize=fs*0.8)
        ax.set_ylabel("Relative Flux",fontsize=fs*0.8)
        ax.tick_params(axis='x', labelsize= fs*0.6)
        ax.tick_params(axis='y', labelsize= fs*0.6)
        ax.set_xlim((min(data["time_array"][filtername_list[i]][f"exp_t{exp_label} [sec]"])-data["mid_time"])*24*1.05,
                    (max(data["time_array"][filtername_list[i]][f"exp_t{exp_label} [sec]"])-data["mid_time"])*24*1.05)
        ax.set_ylim(ymin, ymax)

        ax.legend(loc = 'lower right', prop={'size': fs*0.5})
    
    # check if there is the save folder
    if not os.path.isdir(f"Result/{target_name}/light_curve/{target_date}"):
        print(f"create the folder under path:Result/{target_name}/light_curve/{target_date}")
        os.makedirs(f"Result/{target_name}/light_curve/{target_date}")
    
    if save:
        plt.savefig(f"{filename}.png")
        
    plt.show()
    
    return


def plot_synlc_vsexp(data, exposure_list,
                     mode, filtername,
                     filename,
                     ymax, ymin, save=True):
    """
    <<input>>
    data            : dictionary ; synthetic light curve data            ; generated from LC_syn_tk.py
    exposure_list   : list       ; different exposure time list 
    mode            : string     ; which type of mode will be plotted    ; consider "whole tranist time"/"observable time"
    filtername      : string     ; the filter you want to compare its exposure time
    filename        : string     ; the figure saving path and name
    ymax            : float      ; the y-axis max
    ymin            : float      ; the y-axis min
    save            : boolen     ; whether save the figure or not        ; default= True
    
    <<output>>
    if mode=='obs':
        figure          : figure     ; under path: Result/{target_name}/light_curve/{target_date}/
    
    elif mode=='mod':
        figure          : figure     ; under path: Result/{target_name}/light_curve/
    """
    
    # check the filename and its path
    pathcheck_index  = filename.split('/').index("light_curve")
    pathcheck_length = len(filename.split('/')[pathcheck_index+1:])
    
    if mode=="mod" and pathcheck_length==1:
        title   = "model"
    elif mode=="obs" and pathcheck_length==2:
        title   = "observable"
    else:
        print("The filename has something wrong!")
        """
        if mode=="obs":
            figure          : figure     ; under path: Result/{target_name}/light_curve/{target_date}/
        elif mode=="mod":
            figure          : figure     ; under path: Result/{target_name}/light_curve/
        """
        return
    
    target_name = data["planet_object"].name
    stellar_name = '-'.join(target_name.split('-')[:-1])
    target_date = data["target_date"]
    
    # get the available transit date (by the stardrive datebase)  ; generated from Twinkle Stardrive tool(orbit)
    f = open(f"Data_tk/stardrive/{stellar_name}/{target_name}/transit-results.json")
    tk_avail_time = json.load(f)
    
    # the index of target_date in tk_avail_time
    df = pd.read_csv(f"Data_tk/stardrive/{stellar_name}/{target_name}/{target_name}_transit-date.csv")
    target_lc_num = df["Mid_date"][df["Mid_date"]==target_date].keys()[0]
    
    # plot !!!
    fs=28

    # generate the empty figure
    filtername_list = data["filtername_list"]
    i = filtername_list.index(filtername)
    fig = plt.figure(figsize=(int(10*len(exposure_list[i])/2),12), dpi=200, constrained_layout=True)
    fig.suptitle(f"{target_name} on {target_date} transit in ({title})",fontsize=fs)
    fig.patch.set_facecolor('white')
    
    for j in range(len(exposure_list[i])):
        
        ax = fig.add_subplot(2, int(len(exposure_list[i])/2), j+1)

        exp_label = str(exposure_list[i][j])
        
        # using "mode" parameter to determine which type
        # (mod=whole time / obs=consider earth blocked) of information is the giving data
        if mode=="mod":
            xdata   = data["time_array"][filtername][f"exp_t{exp_label} [sec]"]
            ydata_l = data["tr_mod_array"][filtername][f"exp_t{exp_label} [sec]"]
            ydata_s = data["mod_noise_array"][filtername][f"exp_t{exp_label} [sec]"]

        elif mode=="obs":
            xdata_l = data["time_array"][filtername][str(exposure_list[i][0])]
            xdata_s = data["observable_array"][filtername][f"exp_t{exp_label} [sec]"]
        #     ydata_l = data['tr_obs_array']
            ydata_l = data["tr_mod_array"][filtername][str(exposure_list[i][0])]
            ydata_s = data["obs_noise_array"][filtername][f"exp_t{exp_label} [sec]"]
            
            
        
        ax.set_title(f"exp_time={exp_label}s filter:{filtername}",fontsize=fs*0.8)
        
        if mode=="obs":           
            ax.plot((xdata_l-data["mid_time"])*24,
                     ydata_l,
                     label=filtername,
                     c='k', lw=4.0)
            ax.scatter((xdata_s-data["mid_time"])*24,
                        ydata_s,
                        label=f"exp_time:{exp_label}(s) / Photon noise:"+str(data["err_per_exp [ppm]"][filtername][f"exp_t{exp_label} [sec]"])+"(ppm)",
                        c='C'+str(i), marker='o', s=100)
            
            # because the figure x axis is detemined as -time ~ mid_time=0 ~ +time
            # we should transform the orbit windows information in the same information type
            # (i.e. use the mid_time to be the base of each orbit window)
            t_start  = Time(tk_avail_time["transit"][target_lc_num]["start_time"])
            t_end    = Time(tk_avail_time["transit"][target_lc_num]["end_time"])

            t_mid    = Time((t_start.jd + t_end.jd)/2., format='jd')

            for ii in range(len(tk_avail_time["transit"][target_lc_num]["viewing_times"])):
                view_start  = Time(tk_avail_time["transit"][target_lc_num]["viewing_times"][ii][0])
                view_end    = Time(tk_avail_time["transit"][target_lc_num]["viewing_times"][ii][1])

                view_start  = (view_start.jd - t_mid.jd)*24
                view_end    = (view_end.jd   - t_mid.jd)*24

                if ii == len(tk_avail_time["transit"][target_lc_num]["viewing_times"])-1:
                    ax.fill_between([view_start,view_end],[0.1,0.1],[1.5,1.5],color='b',alpha=0.1,
                                     label="channel converage="+round(tk_avail_time["transit"][target_lc_num]["coverage"]*100,3)+"%")
                else:
                    ax.fill_between([view_start,view_end],[0.1,0.1],[1.5,1.5],color='b',alpha=0.1)
        
        elif mode=='mod':
            ax.plot((xdata-data["mid_time"])*24,
                     ydata_l,
                     label=filtername,
                     c='k', lw=4.0)
            ax.scatter((xdata-data["mid_time"])*24,
                        ydata_s,
                        label=f"exp_time:{exp_label}(s) / Photon noise:"+str(data["err_per_exp [ppm]"][filtername][f"exp_t{exp_label} [sec]"])+"(ppm)",
                        c='C'+str(i), marker='o')

        ax.set_xlabel("Time Since Mid Transit [hours]",fontsize=fs*0.8)
        ax.set_ylabel("Relative Flux",fontsize=fs*0.8)
        ax.tick_params(axis='x', labelsize= fs*0.6)
        ax.tick_params(axis='y', labelsize= fs*0.6)
        ax.set_xlim((min(data["time_array"][filtername][f"exp_t{exp_label} [sec]"])-data["mid_time"])*24*1.05,
                    (max(data["time_array"][filtername][f"exp_t{exp_label} [sec]"])-data["mid_time"])*24*1.05)
        ax.set_ylim(ymin, ymax)

        ax.legend(loc = 'lower right', prop={'size': fs*0.5})
    
    # check if there is the save folder
    if not os.path.isdir(f"Result/{target_name}/light_curve/{target_date}"):
        print(f"create the folder under path:Result/{target_name}/light_curve/{target_date}")
        os.makedirs(f"Result/{target_name}/light_curve/{target_date}")
    
    if save:
        plt.savefig(f"{filename}.png")
        
    plt.show()
    
    return
    
    
    
def plot_synlc_vsbin(data, data_bin, binsize_list,
                     mode, filtername, exp_t,
                     filename,
                     ymax, ymin, save=True):
    
    """
    <<input>>
    data            : dictionary ; synthetic light curve data                    ; generated from LC_syn_tk.py
    data_bin        : dictionary ; synthetic light curve data with data binning  ; generated from LC_syn_tk.py
    binsize_list    : list       ; different binsize list 
    mode            : string     ; which type of mode will be plotted    ; consider "whole tranist time"/"observable time"
    filtername      : string     ; the filter you want to compare its exposure time
    exp_t           : float      ; the exopsure time we want to consider
    filename        : string     ; the figure saving path and name
    ymax            : float      ; the y-axis max
    ymin            : float      ; the y-axis min
    save            : boolen     ; whether save the figure or not        ; default= True
    
    <<output>>
    if mode=='obs':
        figure          : figure     ; under path: Result/{target_name}/light_curve/{target_date}/
    
    elif mode=='mod':
        figure          : figure     ; under path: Result/{target_name}/light_curve/
    """
    
    # check the filename and its path
    pathcheck_index  = filename.split('/').index("light_curve")
    pathcheck_length = len(filename.split('/')[pathcheck_index+1:])
    
    if mode=="mod" and pathcheck_length==1:
        title   = "model" 
    elif mode=="obs" and pathcheck_length==2:
        title   = "observable"
    else:
        print("The filename has something wrong!")
        """
        if mode=="obs":
            figure          : figure     ; under path: Result/{target_name}/light_curve/{target_date}/
        elif mode=="mod":
            figure          : figure     ; under path: Result/{target_name}/light_curve/
        """
        return
    
    target_name = data["planet_object"].name
    target_date = data["target_date"]
    
    # get the available transit date (by the stardrive datebase)  ; generated from Twinkle Stardrive tool(orbit)
    f = open(f"Data_tk/stardrive/{target_name}/transit-results.json")
    tk_avail_time = json.load(f)
    
    # the index of target_date in tk_avail_time
    df = pd.read_csv(f"Data_tk/stardrive/{target_name}/{target_name}_transit-date.csv")
    target_lc_num = df["Mid_date"][df["Mid_date"]==target_date].keys()[0]
    
    # plot !!!
    fs=28

    # generate the empty figure
    filtername_list = data["filtername_list"]
    exposure_list   = data["exposure_list"]
    
    filter_index = filtername_list.index(filtername)
    exp_label = str(exp_t)

    fig = plt.figure(figsize=(int(10*len(binsize_list)/2),12), dpi=200, constrained_layout=True)
    fig.suptitle(f"{target_name} on {target_date} in ({title}) transit with {filtername} in exposure time:{exp_t}(s) \n using differnt data binning size",fontsize=fs)
    fig.patch.set_facecolor('white')
    
    xdata_l = data["time_array"][filtername][str(exposure_list[filter_index][0])]
    ydata_l = data["tr_mod_array"][filtername][str(exposure_list[filter_index][0])]
    
    
    if mode=="mod": 
        xdata_s = data["time_array"][filtername][f"exp_t{exp_label} [sec]"]
        ydata_s = data["mod_noise_array"][filtername][f"exp_t{exp_label} [sec]"]
        
    elif mode=="obs":
        xdata_s = data["observable_array"][filtername][f"exp_t{exp_label} [sec]"]
        ydata_s = data["obs_noise_array"][filtername][f"exp_t{exp_label} [sec]"]
    
    
    for i in range(len(binsize_list)):
        ax = fig.add_subplot(2, int(len(binsize_list)/2), i+1)
        
        
        # using "mode" parameter to determine which type
        # (mod=whole time / obs=consider earth blocked) of information is the giving data
        if mode=="mod":
            xdata   = data["time_array"][filtername][f"exp_t{exp_label} [sec]"]
            ydata_l = data["tr_mod_array"][filtername][f"exp_t{exp_label} [sec]"]
            ydata_b = data["mod_noise_array"][filtername][f"exp_t{exp_label} [sec]"]

        elif mode=="obs":
            if data_bin[str(binsize_list[i])][filtername][f"exp_t{exp_label} [sec]"]:
                xdata_b = data["observable_array"][filtername][f"exp_t{exp_label} [sec]"]
                ydata_b = data["obs_noise_array"][filtername][f"exp_t{exp_label} [sec]"]
            else:
                xdata_b = data_bin[f"lc_b{binsize_list[i]}_obs"][filtername][f"exp_t{exp_label} [sec]"].time.value
                ydata_b = data_bin[f"lc_b{binsize_list[i]}_obs"][filtername][f"exp_t{exp_label} [sec]"].flux.value
        
        ax.set_title(f"data binning size={binsize_list[i]}(min) in exp_time={exp_label}s",fontsize=fs*0.8)
        
        if mode=="obs":           
            ax.plot((xdata_l-data["mid_time"])*24,
                     ydata_l,
                     label="model light curve",
                     c='k', lw=4.0)
            
            ax.scatter((xdata_s-data["mid_time"])*24,
                        ydata_s,
                        label=f"origin data / exp_time:{exp_label}(s)",
                        c='grey', marker='o')
            
            ax.scatter((xdata_b-data["mid_time"])*24,
                        ydata_b,
                        label=f"data binning size={binsize_list[i]}(min)",
                        c='red', marker='o', s=200)
            
            # because the figure x axis is detemined as -time ~ mid_time=0 ~ +time
            # we should transform the orbit windows information in the same information type
            # (i.e. use the mid_time to be the base of each orbit window)
            t_start  = Time(tk_avail_time["transit"][target_lc_num]["start_time"])
            t_end    = Time(tk_avail_time["transit"][target_lc_num]["end_time"])

            t_mid    = Time((t_start.jd + t_end.jd)/2., format='jd')

            for ii in range(len(tk_avail_time["transit"][target_lc_num]["viewing_times"])):
                view_start  = Time(tk_avail_time["transit"][target_lc_num]["viewing_times"][ii][0])
                view_end    = Time(tk_avail_time["transit"][target_lc_num]["viewing_times"][ii][1])

                view_start  = (view_start.jd - t_mid.jd)*24
                view_end    = (view_end.jd   - t_mid.jd)*24

                if ii == len(tk_avail_time["transit"][target_lc_num]["viewing_times"])-1:
                    ax.fill_between([view_start,view_end],[0.1,0.1],[1.5,1.5],color='b',alpha=0.1,
                                     label="channel converage="+round(tk_avail_time["transit"][target_lc_num]["coverage"]*100,3)+"%")
                else:
                    ax.fill_between([view_start,view_end],[0.1,0.1],[1.5,1.5],color='b',alpha=0.1)
        
        elif mode=="mod":
            ax.plot((xdata-data["mid_time"])*24,
                     ydata_l,
                     label=filtername,
                     c='k', lw=4.0)
            ax.scatter((xdata-data["mid_time"])*24,
                        ydata_s,
                        label=f"exp_time:{exp_label}(s) / Photon noise:"+str(data["err_per_exp [ppm]"][filtername][f"exp_t{exp_label} [sec]"])+"(ppm)",
                        c="red", marker='o', s=200)

        ax.set_xlabel("Time Since Mid Transit [hours]",fontsize=fs*0.8)
        ax.set_ylabel("Relative Flux",fontsize=fs*0.8)
        ax.tick_params(axis='x', labelsize= fs*0.6)
        ax.tick_params(axis='y', labelsize= fs*0.6)
        ax.set_xlim((min(data["time_array"][filtername][f"exp_t{exp_label} [sec]"])-data["mid_time"])*24*1.05,
                    (max(data["time_array"][filtername][f"exp_t{exp_label} [sec]"])-data["mid_time"])*24*1.05)
        ax.set_ylim(ymin, ymax)

        ax.legend(loc = "lower right", prop={"size": fs*0.5})
    
    # check if there is the save folder
    if not os.path.isdir(f"Result/{target_name}/light_curve/{target_date}"):
        print(f"create the folder under path:Result/{target_name}/light_curve/{target_date}")
        os.makedirs(f"Result/{target_name}/light_curve/{target_date}")
    
    if save:
        plt.savefig(f"{filename}.png")
        
    plt.show()
    
    return

def plot_lc_channel_vsfilter(data, exposure_list, target_date,
                             filtername_list,
                             filtername_allchannellist,
                             filterchannelnum_list,
                             filename,
                             ymax, ymin, save=True):
    """
    <<input>>
    data                      : dictionary ; synthetic light curve data            ; generated from LC_syn_tk.py
    exposure_list             : list       ; different exposure time list 
    target_date               : string     ; the date of available observation     ; determined by LC_syn_tk.py
    filtername_list           : list(str)  ; different filter (original) name
    filtername_allchannellist : list(str)  ; different filter (+channel) name
    filterchannelnum_list     : list(int)  ; the numbers of filter channel
    filename                  : string     ; the figure saving path and name
    ymax                      : float      ; the y-axis max
    ymin                      : float      ; the y-axis min
    save                      : boolen     ; whether save the figure or not        ; default = True
    
    <<output>>
    if mode=="obs":
        figure          : figure     ; under path: Result/{target_name}/light_curve/{target_date}/
    
    elif mode=="mod":
        figure          : figure     ; under path: Result/{target_name}/light_curve/
    """
    
    target_name = data["planet_object"].name
    stellar_name= '-'.join(target_name.split('-')[:-1])
    
    # the index of target_date in tk_avail_time
    df = pd.read_csv(f"Data_tk/stardrive/{stellar_name}/{target_name}/{target_name}_transit-date.csv")
    target_lc_num = df['Mid_date'][df['Mid_date']==target_date].keys()[0]
    
    # plot !!!
    fs=28

    # generate the empty figure
    fig = plt.figure(figsize=(30,6), dpi=200, constrained_layout=True)
    fig.suptitle(f"{target_name} on {target_date} transit",fontsize=fs)
    fig.patch.set_facecolor('white')

    for i in range(len(filtername_list)):
        ax = fig.add_subplot(1, len(filtername_list), i+1)
        
        if filterchannelnum_list[i] == "default" and filtername_list[i] == "Twinkle_ch0_filter":
            # the color map with gradually change (.jet is one kind of cmap)
            colors = plt.cm.jet(np.linspace(0,1,59))  
            
        elif  filterchannelnum_list[i] == "default" and filtername_list[i] == "Twinkle_ch1_filter":
            # the color map with gradually change (.jet is one kind of cmap)
            colors = plt.cm.jet(np.linspace(0,1,24))
            
        else:
            colors = plt.cm.jet(np.linspace(0,1,filterchannelnum_list[i]))
        
        color_index = -1
        for j in range(len(filtername_allchannellist)):
            filtername_now     = filtername_list[i]
            filter_channel_now = filtername_allchannellist[j].split('-')

            if not filtername_now == filter_channel_now[0]:
                continue
            
            exp_label = str(exposure_list[i][0])

            # using "mode" parameter to determine which type
            # (mod=whole time / obs=consider earth blocked) of information is the giving data
            xdata   = data["time_array"][filtername_allchannellist[j]][f"exp_t{exp_label} [sec]"]
            ydata_l = data["tr_mod_array"][filtername_allchannellist[j]][f"exp_t{exp_label} [sec]"]   

            ax.set_title(f"exp_time={exp_label}s filter:{filtername_list[i]}",fontsize=fs*0.8)

            if not len(filtername_allchannellist[j].split('-'))==1:
                low_wv  = round(float(filter_channel_now[1].split('_')[0])/1e4,2)
                high_wv = round(float(filter_channel_now[1].split('_')[1])/1e4,2)
                
                ax.plot((xdata-data["mid_time"])*24, ydata_l,
                         label=f"{low_wv}-{high_wv}(um)",
                         c=colors[color_index], lw=1.0)
            
            else:
                ax.plot((xdata-data["mid_time"])*24, ydata_l,
                         label="white band", ls='--',
                         c='k', lw=3.0, zorder=100)
            
            color_index = color_index+1

            ax.set_xlabel("Time Since Mid Transit [hours]",fontsize=fs*0.8)
            ax.set_ylabel("Relative Flux",fontsize=fs*0.8)
            ax.tick_params(axis='x', labelsize= fs*0.6)
            ax.tick_params(axis='y', labelsize= fs*0.6)
            ax.set_xlim((min(data["time_array"][filtername_list[i]][f"exp_t{exp_label} [sec]"])-data["mid_time"])*24*1.05,
                        (max(data["time_array"][filtername_list[i]][f"exp_t{exp_label} [sec]"])-data["mid_time"])*24*1.05)
            ax.set_ylim(ymin, ymax)

            ax.legend(loc = 'lower right', prop={'size': fs*0.25}, ncol=2)
            
    # check if there is the save folder
    if not os.path.isdir(f"Result/{target_name}/light_curve"):
        print(f"create the folder under path:Result/{target_name}/light_curve")
        os.makedirs(f"Result/{target_name}/light_curve")
    
    if save:
        plt.savefig(f"{filename}.png")
        
    plt.show()
    
    return


def plot_lc_channel_vsexp(data, exposure_list, target_date,
                         filtername_list,
                         filtername_allchannellist,
                         filterchannelnum_list,
                         filename,
                         ymax, ymin, save=True):
    """
    <<input>>
    data                      : dictionary ; synthetic light curve data            ; generated from LC_syn_tk.py
    exposure_list             : list       ; different exposure time list 
    target_date               : string     ; the date of available observation     ; determined by LC_syn_tk.py
    filtername_list           : list(str)  ; different filter (original) name
    filtername_allchannellist : list(str)  ; different filter (+channel) name
    filterchannelnum_list     : list(int)  ; the numbers of filter channel
    filename                  : string     ; the figure saving path and name
    ymax                      : float      ; the y-axis max
    ymin                      : float      ; the y-axis min
    save                      : boolen     ; whether save the figure or not        ; default= True
    
    <<output>>
    figure          : figure     ; under path: Result/{target_name}/light_curve/
    """
    
    target_name = data["planet_object"].name
    stellar_name= '-'.join(target_name.split('-')[:-1])
    
    # the index of target_date in tk_avail_time
    df = pd.read_csv(f"Data_tk/stardrive/{stellar_name}/{target_name}/{target_name}_transit-date.csv")
    target_lc_num = df['Mid_date'][df['Mid_date']==target_date].keys()[0]
    
    # plot !!!
    fs=28

    # generate the empty figure
    fig = plt.figure(figsize=(30,6), dpi=200, constrained_layout=True)
    fig.suptitle(f"{target_name} on {target_date} transit",fontsize=fs)
    fig.patch.set_facecolor('white')

    for i in range(len(filtername_list)):
        ax = fig.add_subplot(1, len(filtername_list), i+1)
        
        if filterchannelnum_list[i] == "default" and filtername_list[i] == "Twinkle_ch0_filter":
            # the color map with gradually change (.jet is one kind of cmap)
            colors = plt.cm.jet(np.linspace(0,1,59))  
            
        elif  filterchannelnum_list[i] == "default" and filtername_list[i] == "Twinkle_ch1_filter":
            # the color map with gradually change (.jet is one kind of cmap)
            colors = plt.cm.jet(np.linspace(0,1,24))
            
        else:
            colors = plt.cm.jet(np.linspace(0,1,filterchannelnum_list[i]))
        
        color_index = -1
        exp_label = str(exposure_list[i][0])
        
        for j in range(len(filtername_allchannellist)):
            filtername_now     = filtername_list[i]
            filter_channel_now = filtername_allchannellist[j].split('-')

            if not filtername_now == filter_channel_now[0]:
                continue
            
            # using "mode" parameter to determine which type
            # (mod=whole time / obs=consider earth blocked) of information is the giving data
            xdata   = data["time_array"][filtername_allchannellist[j]][f"exp_t{exp_label} [sec]"]
            ydata_l = data["tr_mod_array"][filtername_allchannellist[j]][f"exp_t{exp_label} [sec]"]   

            ax.set_title(f"exp_time={exp_label}s filter:{filtername_list[i]}",fontsize=fs*0.8)

            if not len(filtername_allchannellist[j].split('-'))==1:
                low_wv  = round(float(filter_channel_now[1].split('_')[0])/1e4,2)
                high_wv = round(float(filter_channel_now[1].split('_')[1])/1e4,2)
                
                ax.plot((xdata-data["mid_time"])*24, ydata_l,
                         label=f"{low_wv}-{high_wv}[um]",
                         c=colors[color_index], lw=1.0)
            
            else:
                ax.plot((xdata-data["mid_time"])*24, ydata_l,
                         label="White Band", ls='--',
                         c='k', lw=3.0, zorder=100)
            
            color_index = color_index+1

            ax.set_xlabel("Time Since Mid Transit [hours]",fontsize=fs*0.8)
            ax.set_ylabel("Relative Flux",fontsize=fs*0.8)
            ax.tick_params(axis='x', labelsize= fs*0.6)
            ax.tick_params(axis='y', labelsize= fs*0.6)
            ax.set_xlim((min(data["time_array"][filtername_list[i]][f"exp_t{exp_label} [sec]"])-data["mid_time"])*24*1.05,
                        (max(data["time_array"][filtername_list[i]][f"exp_t{exp_label} [sec]"])-data["mid_time"])*24*1.05)
            ax.set_ylim(ymin, ymax)

            ax.legend(loc = 'lower right', prop={'size': fs*0.25}, ncol=2)
            
    # check if there is the save folder
    if not os.path.isdir(f"Result/{target_name}/light_curve"):
        print(f"create the folder under path:Result/{target_name}/light_curve")
        os.makedirs(f"Result/{target_name}/light_curve")
    
    if save:
        plt.savefig(f"{filename}.png")
        
    plt.show()
    
    return


def plot_lc_channel_offset(data, target_date, exp_t,
                           filtername,
                           filtername_list,
                           filtername_allchannellist,
                           filterchannelnum_list,
                           filename,
                           offset,
                           save=True):
    """
    <<input>>
    data                      : dictionary ; synthetic light curve data            ; generated from LC_syn_tk.py
    target_date               : string     ; the date of available observation     ; determined by LC_syn_tk.py
    exp_t                     : float      ; the exposure time we want to do plot
    filtername                : string     ; filter (original) name
    filtername_list           : list(str)  ; different filter (original) name
    filtername_allchannellist : list(str)  ; different filter (+channel) name
    filterchannelnum_list     : list(int)  ; the numbers of filter channel for each filter
    filename                  : string     ; the figure saving path and name
    offset                    : float      ; 
    save                      : boolen     ; whether save the figure or not        ; default= True
    
    <<output>>
    figure                    : figure     ; under path: Result/{target_name}/light_curve/
    """
    
    target_name   = data["planet_object"].name
    stellar_name  = '-'.join(target_name.split('-')[:-1])
    
    # the index of target_date in tk_avail_time
    df = pd.read_csv(f"Data_tk/stardrive/{stellar_name}/{target_name}/{target_name}_transit-date.csv")
    target_lc_num = df['Mid_date'][df['Mid_date']==target_date].keys()[0]
    
    # plot !!!
    fs=24

    # generate the empty figure
    i = filtername_list.index(filtername)

    if filterchannelnum_list[i] == "default" and filtername == "Twinkle_ch0_filter":
        # the color map with gradually change (.jet is one kind of cmap)
        colors       = plt.cm.jet(np.linspace(0,1,59))

    elif  filterchannelnum_list[i] == "default" and filtername == "Twinkle_ch1_filter":
        # the color map with gradually change (.jet is one kind of cmap)
        colors       = plt.cm.jet(np.linspace(0,1,24))
            
    else:
        colors       = plt.cm.jet(np.linspace(0,1,filterchannelnum_list[i]))
    
    fig = plt.figure(figsize=(12,1*len(colors)), dpi=200, constrained_layout=True)
    fig.suptitle(f"{target_name} in smaller wavelength interval",fontsize=fs)
    fig.patch.set_facecolor('white')
    ax = fig.add_subplot(1, 1, 1)    
    
        
    color_index  = -1
    offset_index = 0
    ymin         = 1
    yticks       = list()
    yticks_yaxis = list()
    
    for j in range(len(filtername_allchannellist)):
        filter_channel_now = filtername_allchannellist[j].split('-')

        if not filtername == filter_channel_now[0]:
            continue

        # using "mode" parameter to determine which type
        # (mod=whole time / obs=consider earth blocked) of information is the giving data
        xdata_s = data["time_array"][filtername_allchannellist[j]][f"exp_t{exp_t} [sec]"]
        ydata_s = data["mod_noise_array"][filtername_allchannellist[j]][f"exp_t{exp_t} [sec]"]-offset_index*offset
        xdata_l = data["time_array"][filtername_allchannellist[j]][f"exp_t60 [sec]"]
        ydata_l = data["tr_mod_array"][filtername_allchannellist[j]][f"exp_t60 [sec]"]-offset_index*offset
        
        ax.set_title(f"Exp_time={exp_t} [sec] / filter:{filtername} dividing into {len(colors)} channels",fontsize=fs*0.8)

        if not len(filtername_allchannellist[j].split('-'))==1:
            low_wv  = round(float(filter_channel_now[1].split('_')[0])/1e4,2)
            high_wv = round(float(filter_channel_now[1].split('_')[1])/1e4,2)

            ax.scatter((xdata_s-data["mid_time"])*24, ydata_s,
                     label=f"{low_wv}-{high_wv}[um]",
                     color=colors[color_index])
            
            ax.plot((xdata_l-data["mid_time"])*24, ydata_l,
                     ls='-', color=colors[color_index])
            yticks.append(f"{low_wv}-{high_wv}[um]")
            yticks_yaxis.append(ydata_l[0])

        else:
            ax.scatter((xdata_s-data["mid_time"])*24, ydata_s,
                     label="White Band", color='k')
            ax.plot((xdata_l-data["mid_time"])*24, ydata_l,
                     ls='-', color='k')
            yticks.append("White Band")
            yticks_yaxis.append(ydata_l[0])

        color_index  = color_index+1
        offset_index = offset_index+1
        ymin_each    = np.min(ydata_s)
        ymin         = min(ymin_each, ymin)

    ax.set_xlabel("Time Since Mid Transit [hours]",fontsize=fs*0.8)
    ax.set_ylabel("Relative Flux",fontsize=fs*0.8)
    ax.tick_params(axis='x', labelsize= fs*0.6)
    ax.tick_params(axis='y', labelsize= fs*0.6)
    ax.set_xlim((min(data["time_array"][filtername][f"exp_t{exp_t} [sec]"])-data["mid_time"])*24*1.05,
                (max(data["time_array"][filtername][f"exp_t{exp_t} [sec]"])-data["mid_time"])*24*1.05)
    ax.set_ylim(ymin-0.005, 1.005)
        
    secax = ax.secondary_yaxis("right")
    secax.set_ylabel("Wavelength [um]",fontsize=fs*0.8)
    secax.set_yticks(yticks_yaxis, labels=yticks, fontsize=fs*0.6)
    # ax.secondary_yaxis(yticks_yaxis, labels=yticks, fontsize=fs*0.6, secondary_y=True)
            
    # check if there is the save folder
    if not os.path.isdir(f"Result/{target_name}/light_curve"):
        print(f"create the folder under path:Result/{target_name}/light_curve")
        os.makedirs(f"Result/{target_name}/light_curve")
    
    if save:
        plt.savefig(f"{filename}.png")
        
    plt.show()
    
    return


#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import os, copy, pickle
import numpy as np
import pandas as pd
# import pylightcurve as plc
from pylightcurve_master import pylightcurve_self as plc

# The package for parallel computing
import multiprocessing

# In[ ]:


def add_observation(planet,
                    exp_t,
                    time_array,
                    flux_array,
                    flux_unc_array,
                    filtername):
    
    planet.add_observation(time = time_array,
                    time_format = 'BJD_TDB',   
                    exp_time = exp_t,
                    time_stamp = 'mid', 
                    flux = flux_array, 
                    flux_unc = flux_unc_array,   
                    flux_format = 'flux',
                    filter_name = filtername)


# In[ ]:


def run_fitting(planet, date, exp_t, bin_size, filtername, mode):
    
    """
    <<input>>
    planet     : pylightcurve planet object ; synthetic light curve data ; generated from func:create_planet_tkdata
    date       : string                     ; observation date           ; the chosen date of available observation by Starfrive tool
    exp_t      : int                        ; exposure time              ; will be determined in LC_fitting.py
    binsize    : int                        ; bin size
    filtername : string                     ; filter name 
    mode       : string                     ; consider "whole time"/"observable time" ; 'mod' / 'obs']
    
    <<output>>
    **         : running                    ; execute the fitting code   ; generated the mcmc_fitting
    """
    
    save_planet  = planet.name
    save_stellar = '-'.join(save_planet.split('-')[:-1])
    # check if there is the save folder
    if not os.path.isdir(f"Savefile/{save_stellar}/{save_planet}/{date}/fitting/{bin_size}"):
        os.makedirs(f"Savefile/{save_stellar}/{save_planet}/{date}/fitting/{bin_size}")
        
    # path of saving file
    save_path = f"Savefile/{save_stellar}/{save_planet}/{date}/fitting/{bin_size}/{bin_size}_t{exp_t}_{mode}_{filtername}_fit"
    
    print("Fitting result save in:")
    print(save_path)
    
#     planet.transit_fitting(save_path,
#                       detrending_order=1,
#                       iterations=50000,
#                       burn_in = 25000,
#                       walkers=50,
#                       fit_rp_over_rs=True,
#                       fit_mid_time=True)
    
    planet.transit_fitting(save_path,
                      detrending_order=1,
                      iterations=100000,
                      burn_in = 50000,
                      walkers=50,
                      fit_mid_time=True)


def lc_fitting(target_name, target_date,
               mode_list         = ["mod", "obs"],
               run_nonbinned_fit = True,
               run_binning_fit   = True,
               multiple_process  = False,
               avail_cpus        = int(os.cpu_count()/2)):

    """
    <<input>>
    target_name       : string       ; target planet_name                      
    planet_date       : string       ; target planet_date                        ; list in LC_syn_tk.ipynb
    mode_list         : list(string) ; consider "whole time"/"observable time"   ; default = ['mod', 'obs']
    run_nonbinned_fit : bool         ; whether run non-binned_fit fitting or not ; default = True
    run_binning_fit   : bool         ; whether run binning fitting or not        ; default = True
    multiple_process  : bool         ; whether simultaneously run differnt exp_t ; default = False 
    
    <<output>>
    **              : running      ; execute the fitting code                ; define in same file func_fitv2.py
    """

    stellar_name = '-'.join(target_name.split('-')[:-1])
    
    data     = pickle.load(open(f"Savefile/{stellar_name}/{target_name}/{target_date}/time_flux/{target_name}_{target_date}.pickle",'rb'))
    data_bin = pickle.load(open(f"Savefile/{stellar_name}/{target_name}/{target_date}/time_flux/{target_name}_bin_{target_date}.pickle",'rb'))

    date            = data["target_date"]
    planet_tk       = data["planet_object"]
    exposure_list   = data["exposure_list [sec]"]
    filtername_list = data["filtername_list"]   # 'Twinkle_ch0_filter','Twinkle_ch1_filter'
    print(f"The filter list is: {filtername_list}")
    
    binsize_list    = data_bin["binsize_list [min]"]
    
    # check the date from the saving path is the same as the synthetic object
    if not target_date==date:
        print("Something go wrong QQ")
        return
    
    for mode in mode_list:
        index = 0
        for filtername in filtername_list:
            for exp_t in exposure_list[index]:
                planet = copy.deepcopy(planet_tk)

                if mode == "mod":
                    time_array     = data["time_array"][filtername][f"exp_t{exp_t} [sec]"]
                    flux_array     = data["mod_noise_array"][filtername][f"exp_t{exp_t} [sec]"]
                    flux_unc_array = np.full((len(data["mod_noise_array"][filtername][f"exp_t{exp_t} [sec]"]),),
                                             data["err_per_exp [ppm]"][filtername][f"exp_t{exp_t} [sec]"]/1e6)

                elif mode == "obs":
                    time_array     = data["observable_array"][filtername][f"exp_t{exp_t} [sec]"]
                    flux_array     = data["obs_noise_array"][filtername][f"exp_t{exp_t} [sec]"]
                    flux_unc_array = np.full((len(data["obs_noise_array"][filtername][f"exp_t{exp_t} [sec]"]),),
                                             data["err_per_exp [ppm]"][filtername][f"exp_t{exp_t} [sec]"]/1e6)

                add_observation(planet,
                                exp_t,
                                time_array,
                                flux_array,
                                flux_unc_array,
                                filtername)

                print("Do fitting of non-binning data in date:")
                print(f"{date} exp_t:{exp_t} datatype:{mode} filter:{filtername}")

                if run_nonbinned_fit:
                    # For parallel computing
                    # We use the function of "multiprocessing.Process" to let our function "run_fitting" become one of the task.
                    # Then, we use .start() to let the CPU start to execute the task, and continue to do other tasks if we need.
                    # We can also use .join to let CPU wait for the previous tasks finish.
                    if multiple_process:
                        pool =  multiprocessing.Pool(processes = avail_cpus)
                        pool.apply_async(run_fitting, args=(planet, date, exp_t, "non-binned", filtername, mode))
                    else:
                        run_fitting(planet, date, exp_t, "non-binned", filtername, mode)
            
            index = index + 1
            
    # for multiple_process, wait until the non-binned data finish
    if run_nonbinned_fit and multiple_process:
        pool.close()
        pool.join()

    for bin_size in binsize_list:

        for mode in mode_list:
            index = 0
            for filtername in filtername_list:
                for exp_t in exposure_list[index]:
                    planet_b = copy.deepcopy(planet_tk)

                    if data_bin[f"check_expvsbin_t{bin_size} [min]"][filtername][f"exp_t{exp_t} [sec]"]:
                        print(f"The bin_size{bin_size}(min) > exposure_time{exp_t}(sec), skip binning fitting")
                        continue

                    if mode == "mod":
                        time_array     = data_bin[f"lc_b{bin_size}_mod [min]"][filtername][f"exp_t{exp_t} [sec]"].time.value
                        flux_array     = data_bin[f"lc_b{bin_size}_mod [min]"][filtername][f"exp_t{exp_t} [sec]"].flux.value
                        flux_unc_array = data_bin[f"lc_b{bin_size}_mod [min]"][filtername][f"exp_t{exp_t} [sec]"].flux_err.value/1e6

                    elif mode == "obs":
                        time_array     = data_bin[f"lc_b{bin_size}_obs [min]"][filtername][f"exp_t{exp_t} [sec]"].time.value
                        flux_array     = data_bin[f"lc_b{bin_size}_obs [min]"][filtername][f"exp_t{exp_t} [sec]"].flux.value
                        flux_unc_array = data_bin[f"lc_b{bin_size}_obs [min]"][filtername][f"exp_t{exp_t} [sec]"].flux_err.value/1e6

                        time_array     = time_array[np.isfinite(flux_array)]
                        flux_array     = flux_array[np.isfinite(flux_array)]
                        flux_unc_array = flux_unc_array[np.isfinite(flux_unc_array)]             

                    add_observation(planet_b,
                                    exp_t,
                                    time_array,
                                    flux_array,
                                    flux_unc_array,
                                    filtername)

                    print(f"Do fitting of {bin_size}bin data in date:")
                    print(f"{date} exp_t:{exp_t} datatype:{mode} filter:{filtername}")

                    if run_binning_fit:
                        # For parallel computing
                        # We use the function of "multiprocessing.Process" to let our function "run_fitting" become one of the task.
                        # Then, we use .start() to let the CPU start to execute the task, and continue to do other tasks if we need.
                        # We can also use .join to let CPU wait for the previous tasks finish.
                        if multiple_process:
                            pool =  multiprocessing.Pool(processes = avail_cpus)
                            pool.apply_async(run_fitting, args=(planet_b, date, exp_t, 'bin'+str(bin_size), filtername, mode))
                        else:
                            run_fitting(planet_b, date, exp_t, 'bin'+str(bin_size), filtername, mode)
                    
                index = index + 1
        
    # for multiple_process, wait until the previos task become finish.
    if run_binning_fit and multiple_process:
        pool.close()
        pool.join()



import pickle, json
import numpy as np
import pandas as pd
import lightkurve as lk


def binning(data, binsize_list, exposure_list, filtername_list):
    """
    <<input>>
    data            : dictionary   ; synthetic light curve data   ; generated from LC_syn_tk.py
    binsize_list    : list(int)    ; different bin size 
    exposure_list   : list(int)    ; different exposure time list 
    filtername_list : list(string) ; different filter name 
    
    <<output>>
    data            : dictionary ; synthetic light curve data   ; generated from LC_syn_tk.py
                      structure  : data_bin[
                                            lc_b5_mod,
                                            lc_b5_obs,
                                            syn_b5_mod,
                                            syn_b5_obs[
                                                       10,
                                                       20,
                                                       30[
                                                          filter1,
                                                          filter2[
                                                                  Lightkurve object
                                                                 ],
                                                          ],
                                                       ...
                                                      ],
                                            ...
                                           ]
    """
    target_name = data["planet_object"].name
    stellar_name= '-'.join(target_name.split('-')[:-1])
    
    target_date = data["target_date"]
    mid_time    = data["mid_time"]
    time        = data["time_array"]
    obs_time    = data["observable_array"]
    orbit_time  = data["orbit_array"]
    err_per_exp = data["err_per_exp [ppm]"]

    tr_mod      = data["tr_mod_array"]
    tr_obs      = data["tr_obs_array"]
    mod_noise   = data["mod_noise_array"]
    obs_noise   = data["obs_noise_array"]

    lc_mod  = {}
    lc_obs  = {}
    syn_mod = {}
    syn_obs = {}


    # get the num of observable channel on the certain date of this target
    #############################################################################
    # get the available transit date (by the stardrive datebase)  ; generated from Twinkle Stardrive tool(orbit)
    f = open(f"Data_tk/stardrive/{stellar_name}/{target_name}/transit-results.json")
    tk_avail_time = json.load(f)

    # the index of target_date in tk_avail_time
    df = pd.read_csv(f"Data_tk/stardrive/{stellar_name}/{target_name}/{target_name}_transit-date.csv")
    target_lc_num = df["Mid_date"][df["Mid_date"]==target_date].keys()[0]

    num_orbits = tk_avail_time["transit"][target_lc_num]["num_views"]
    #############################################################################

    filter_index = 0
    # let data become lightkurve object of lightkcurve package
    for filtername in filtername_list:
        lc_mod[filtername]  = []
        syn_mod[filtername] = []

        lc_obs[filtername]  = {}
        syn_obs[filtername] = {}

        for i in range(len(exposure_list[filter_index])):
            exp_t = str(exposure_list[filter_index][i])

            lc_mod[filtername].append(lk.LightCurve(time = time[filtername][f"exp_t{exp_t} [sec]"],
                                                    flux = mod_noise[filtername][f"exp_t{exp_t} [sec]"],
                                                    flux_err = np.full((len(time[filtername][f"exp_t{exp_t} [sec]"]),),
                                                                       err_per_exp[filtername][f"exp_t{exp_t} [sec]"])))
            
            syn_mod[filtername].append(lk.LightCurve(time = time[filtername][f"exp_t{exp_t} [sec]"],
                                                     flux = tr_mod[filtername][f"exp_t{exp_t} [sec]"],
                                                     flux_err = np.full((len(tr_mod[filtername][f"exp_t{exp_t} [sec]"]),),
                                                                        err_per_exp[filtername][f"exp_t{exp_t} [sec]"])))

            lc_obs[filtername][f"exp_t{exp_t} [sec]"]  = {}
            syn_obs[filtername][f"exp_t{exp_t} [sec]"] = {}

            start_point = 0
            for ii in range(int(num_orbits)):
                point_num = len(orbit_time[filtername][f"exp_t{exp_t} [sec]"][f"orbit_{ii}"])
                end_point = start_point + point_num

                lc_obs[filtername][f"exp_t{exp_t} [sec]"][f"orbit_{ii}"] = \
                lk.LightCurve(time = orbit_time[filtername][f"exp_t{exp_t} [sec]"][f"orbit_{ii}"],
                              flux = obs_noise[filtername][f"exp_t{exp_t} [sec]"][start_point:end_point],
                              flux_err = np.full((point_num,), err_per_exp[filtername][f"exp_t{exp_t} [sec]"]))

                syn_obs[filtername][f"exp_t{exp_t} [sec]"][f"orbit_{ii}"] = \
                lk.LightCurve(time = orbit_time[filtername][f"exp_t{exp_t} [sec]"][f"orbit_{ii}"],
                              flux = tr_obs[filtername][f"exp_t{exp_t} [sec]"][start_point:end_point],
                              flux_err = np.full((point_num,),  err_per_exp[filtername][f"exp_t{exp_t} [sec]"]))

                start_point = end_point

        filter_index = filter_index + 1
    
    data_bin = {}
    data_bin["binsize_list [min]"] = binsize_list

    # binning
    for bin_t in binsize_list:  # binning time size is in unit : [min]  
        data_bin[f"check_expvsbin_t{bin_t} [min]"] = {}
        data_bin[f"lc_b{bin_t}_mod [min]"]         = {}
        data_bin[f"lc_b{bin_t}_obs [min]"]         = {}
        data_bin[f"syn_b{bin_t}_mod [min]"]        = {}
        data_bin[f"syn_b{bin_t}_obs [min]"]        = {}       
            
        filter_index = 0
        for filtername in filtername_list:
            data_bin[f"check_expvsbin_t{bin_t} [min]"][filtername] = {}
            data_bin[f"lc_b{bin_t}_mod [min]"][filtername]         = {}    
            data_bin[f"lc_b{bin_t}_obs [min]"][filtername]         = {} 
            data_bin[f"syn_b{bin_t}_mod [min]"][filtername]        = {} 
            data_bin[f"syn_b{bin_t}_obs [min]"][filtername]        = {}
        
            for i in range(len(exposure_list[filter_index])):
                exp_t = str(exposure_list[filter_index][i])
                
                if float(exp_t)/60 > bin_t:
                    data_bin[f"check_expvsbin_t{bin_t} [min]"][filtername][f"exp_t{exp_t} [sec]"] = True
                    continue
                
                else:
                    data_bin[f"check_expvsbin_t{bin_t} [min]"][filtername][f"exp_t{exp_t} [sec]"] = False
                    
                    data_bin[f"lc_b{bin_t}_mod [min]"][filtername][f"exp_t{exp_t} [sec]"]  \
                            = lc_mod[filtername][i].bin(time_bin_size=bin_t/60/24)
                    data_bin[f"syn_b{bin_t}_mod [min]"][filtername][f"exp_t{exp_t} [sec]"] \
                            = syn_mod[filtername][i].bin(time_bin_size=bin_t/60/24)
                    
                    data_bin[f"lc_b{bin_t}_obs [min]"][filtername][f"exp_t{exp_t} [sec]"]   \
                            = lc_combine(lc_obs[filtername][f"exp_t{exp_t} [sec]"],
                                         orbit_time[filtername][f"exp_t{exp_t} [sec]"],
                                         num_orbits, bin_t)
                    data_bin[f"syn_b{bin_t}_obs [min]"][filtername][f"exp_t{exp_t} [sec]"]  \
                            = lc_combine(syn_obs[filtername][f"exp_t{exp_t} [sec]"],
                                         orbit_time[filtername][f"exp_t{exp_t} [sec]"],
                                         num_orbits, bin_t)
                    
                
            filter_index = filter_index + 1
            
    return data_bin

def lc_combine(lc_obs, orbit_time, num_orbits, bin_t):
    time  = np.array([])
    flux  = np.array([])
    error = np.array([])
    
    for ii in range(int(num_orbits)):
        if len(lc_obs[f"orbit_{ii}"])<=1:
            continue
        bin_lc_element = lc_obs[f"orbit_{ii}"].bin(time_bin_size=bin_t/60/24)
        
        orbit_time_check = orbit_time[f"orbit_{ii}"]
        
        flux_time_ckeck_i = np.array(bin_lc_element.time.value)
        flux_flux_ckeck_i = np.array(bin_lc_element.flux.value)
        flux_err_ckeck_i  = np.array(bin_lc_element.flux_err.value)
        
        flux_time_ckeck_f = flux_time_ckeck_i[flux_time_ckeck_i<max(orbit_time_check)]
        flux_flux_ckeck_f = flux_flux_ckeck_i[flux_time_ckeck_i<max(orbit_time_check)]
        flux_err_ckeck_f  = flux_err_ckeck_i[flux_time_ckeck_i<max(orbit_time_check)]
        
        time  = np.append(time, flux_time_ckeck_f)
        flux  = np.append(flux, flux_flux_ckeck_f)
        error = np.append(error, flux_err_ckeck_f)
        
    return lk.LightCurve(time = time, flux = flux, flux_err = error)


import setuptools

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="yu_twinkle",
    version="0.0.2",
    author="Kevin Yu",
    author_email="wmjx691@gmail.com",
    description="Twinkle project functions.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/wmjx691/yu_twinkle",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



import pickle, os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from astropy.time import Time


def get_fitting_result(target_name, target_date, binsize_list, filtername_list, exposure_list):
    """
    <<input>>
    target_name     : string     ; the target name
    target_date     : string     ; the date of available observation     ; determined by LC_syn_tk.py
    binsize_list    : list(int)  ; the binsize list for all binsize
    filtername_list : list(str)  ; different filter name 
    exposure_list   : list(int)  ; different exposure time list 
    
    <<output>>
    fitting result  : dictionary ; getting the fitting result form each fitting result's folder
    """
    
    stellar_name= '-'.join(target_name.split('-')[:-1])
    data_bin = pickle.load(open(f"Savefile/{stellar_name}/{target_name}/{target_date}/time_flux/{target_name}_bin_{target_date}.pickle",'rb'))
    
    data_fitting = {}
    
    # data_fitting["rp"]        = {}
    # data_fitting["rp_low"]    = {}
    # data_fitting["rp_high"]   = {}
    data_fitting["tmid"]      = {}
    data_fitting["tmid_low"]  = {}
    data_fitting["tmid_high"] = {}
    data_fitting["rechi"]     = {}
    
    # for mode in ['mod', 'obs']:
    for mode in ["obs"]:
        # data_fitting["rp"][mode]        = {}
        # data_fitting["rp_low"][mode]    = {}
        # data_fitting["rp_high"][mode]   = {}
        data_fitting["tmid"][mode]      = {}
        data_fitting["tmid_low"][mode]  = {}
        data_fitting["tmid_high"][mode] = {}
        data_fitting["rechi"][mode]     = {}

        for bin_t in binsize_list:  
            # for finding the path of the fitting result
            if bin_t == 0:
                bin_name = "non-binned"
            else:
                bin_name = f"bin{bin_t}"
            # data_fitting["rp'][mode][bin_t]        = {}
            # data_fitting["rp_low'][mode][bin_t]    = {}
            # data_fitting["rp_high"][mode][bin_t]   = {}
            data_fitting["tmid"][mode][bin_t]      = {}
            data_fitting["tmid_low"][mode][bin_t]  = {}
            data_fitting["tmid_high"][mode][bin_t] = {}
            data_fitting["rechi"][mode][bin_t]     = {}

            index = 0
            for filtername in filtername_list:
                # the list to save the information of each exposure time
                # in the same filter/ bin_t
                # rp_t        = []
                # rp_low_t    = []
                # rp_high_t   = []
                tmid_t      = []
                tmid_low_t  = []
                tmid_high_t = []
                rechi_t     = []

                # data_fitting["rp"][mode][bin_t][filtername]        = {}
                # data_fitting["rp_low"][mode][bin_t][filtername]    = {}
                # data_fitting["rp_high"][mode][bin_t][filtername]   = {}
                data_fitting["tmid"][mode][bin_t][filtername]      = {}
                data_fitting["tmid_low"][mode][bin_t][filtername]  = {}
                data_fitting["tmid_high"][mode][bin_t][filtername] = {}
                data_fitting["rechi"][mode][bin_t][filtername]     = {}

                # search the rp/rs&t_mid we want from each exposure time's fitting result               
                for exp_t in exposure_list[index]:
                    if not bin_name == "non-binned":             
                        if data_bin[f"check_expvsbin_t{bin_t} [min]"][filtername][f"exp_t{exp_t} [sec]"]:
                            continue
                    
                    path = f"Savefile/{stellar_name}/{target_name}/{target_date}/fitting/{bin_name}/{bin_name}_t{exp_t}_{mode}_{filtername}_fit/results.txt"
                    df = pd.read_csv(path , sep='\s+')    # the .txt result file

                    # row_rp    = df.loc[df['#'] == 'rp_'+filtername]
                    row_tmid  = df.loc[df['#'] == "T_mid_0"]
                    row_rechi = df.loc[df['#'] == "#Reduced"]

                    # data_fitting['rp'][mode][bin_t][filtername][exp_t]        = float(row_rp['variable'].values)
                    # data_fitting['rp_low'][mode][bin_t][filtername][exp_t]    = float(row_rp['result'].values)
                    # data_fitting['rp_high'][mode][bin_t][filtername][exp_t]   = float(row_rp['uncertainty'].values)
                    data_fitting["tmid"][mode][bin_t][filtername][exp_t]      = float(row_tmid["variable"].values)
                    data_fitting["tmid_low"][mode][bin_t][filtername][exp_t]  = float(row_tmid["result"].values)
                    data_fitting["tmid_high"][mode][bin_t][filtername][exp_t] = float(row_tmid["uncertainty"].values)
                    data_fitting["rechi"][mode][bin_t][filtername][exp_t]     = float(row_rechi["uncertainty"].values)


                index = index + 1
    
    return data_fitting


def gen_fitresult_csv(data_dict, binsize_list, filtername_list, exposure_list, target_name, target_date, real_mid_time):
    """
    <<input>>
    data_dict       : dtaframe   ; the csv file of fthe fitting result   ; generated func:gen_fitresult_csv=
    binsize_list    : list(int)  ; the binsize list for all binsize
    filtername_list : list(str)  ; different filter name 
    exposure_list   : list(int)  ; different exposure time list 
    target_name     : string     ; the target name
    target_date     : string     ; the date of available observation     ; determined by LC_syn_tk.py
    real_mid_time   : float      ; the real mid_time time we given in the Twinkle data
    
    <<output>>
    csv file        : csv dataframe ; under path: Result/{target_name}/fitting/{target_date}/fitting_result_bin_{target_name}_{target_date}.csv
    """

    data_csv = {}
    data_csv = pd.DataFrame(data_csv)

    for mode in ["obs"]:
    # for mode in ['mod', 'obs']:

        # for bin_element in [0]:
        for bin_t in binsize_list:
            for filtername in filtername_list:
                filter_index = filtername_list.index(filtername)

                data_csv_ = {
                    "time_mode"          : "",
                    "binsize [min]"      : "",
                    "exp_t [sec]"        : "",
                    "filtername"         : "",
                    "$\Delta$tmid [min]" : data_dict["tmid"][mode][bin_t][filtername],
                    "tmid_errorbar [min]": "",
                    "tmid_low [min]"     : data_dict["tmid_low"][mode][bin_t][filtername],
                    "tmid_high [min]"    : data_dict["tmid_high"][mode][bin_t][filtername],
                    "tmid [BJD]"         : data_dict["tmid"][mode][bin_t][filtername],
                    # "$\Delta$rp [%]"    : data_dict['rp'][mode][bin_t][filtername],
                    # "rp_errorbar [%]"   : "",
                    # "rp_low [%]"        : data_dict['rp_low'][mode][bin_t][filtername],
                    # "rp_high [%]"       : data_dict['rp_high'][mode][bin_t][filtername],
                    # "rp [rp/rs]"         : data_dict['rp'][mode][bin_t][filtername],
                    "rechi"              : data_dict["rechi"][mode][bin_t][filtername]
                }

                data_csv_ = pd.DataFrame(data_csv_)
                
                
                data_csv_["binsize [min]"] = [bin_t]*len(data_csv_.index)
                data_csv_["exp_t [sec]"]   = data_csv_.index
                data_csv_["time_mode"]     = [mode]*len(data_csv_.index)
                data_csv_["filtername"]    = [filtername]*len(data_csv_.index)


                data_csv_["$\Delta$tmid [min]"]  = round((data_csv_["$\Delta$tmid [min]"]-real_mid_time)*24*60, 3)
                data_csv_["tmid_errorbar [min]"] = round((data_csv_["tmid_high [min]"]+(-data_csv_["tmid_low [min]"]))*24*60, 2)
                data_csv_["tmid_low [min]"]      = round(data_csv_["tmid_low [min]"]*24*60, 2)
                data_csv_["tmid_high [min]"]     = round(data_csv_["tmid_high [min]"]*24*60, 2)
                data_csv_["tmid [BJD]"]          = round(data_csv_["tmid [BJD]"], 5)


                # data_csv_["$\Delta$rp [%]"]     = round(((data_csv_["$\Delta$rp [%]"]-real_rp)/real_rp)*100, 3)
                # data_csv_["rp_errorbar [%]"]    = round(((data_csv_["rp_high [%]"]+(-data_csv_["rp_low [%]"]))/real_rp)*100, 2)
                # data_csv_["rp_low [%]"]         = round((data_csv_["rp_low [%]"]/real_rp)*100, 2)
                # data_csv_["rp_high [%]"]        = round((data_csv_["rp_high [%]"]/real_rp)*100, 2)
                # data_csv_["rp [rp/rs]"]         = round(data_csv_["rp [rp/rs]"], 3)

                data_csv_["rechi"]              = round(data_csv_["rechi"], 3)


                data_csv = pd.concat([data_csv, data_csv_])


    # check if there is the save folder
    if not os.path.isdir(f"Result/{target_name}/fitting/{target_date}"):
        print(f"create the folder under path:Result/{target_name}/fitting/{target_date}")
        os.makedirs(f"Result/{target_name}/fitting/{target_date}")

    data_csv.to_csv(f"Result/{target_name}/fitting/{target_date}/fitting_result_bin_{target_name}_{target_date}.csv", index=False)
    
    return data_csv


def plot_midt_fitting_result(df, target_name, target_date, binsize_list,
                             mode, filtername, real_mid_time,
                             filename,
                             save=True):
    """
    <<input>>
    df              : dtaframe   ; the csv file of fthe fitting result   ; generated func:gen_fitresult_csv
    target_name     : string     ; the target name
    target_date     : string     ; the date of available observation     ; determined by LC_syn_tk.py
    binsize_list    : list(int)  ; the binsize list for all binsize
    mode            : string     ; which type of mode will be plotted    ; consider "whole tranist time"/"observable time"
    filtername      : string     ; the filtername that you want to generate its fitting result figure
    real_mid_time   : float      ; the real mid_time time we given in the Twinkle data
    filename        : string     ; the figure saving path and name
    save            : boolen     ; whether save the figure or not        ; default= True
    
    <<output>>
    figure          : figure     ; under path: Result/{target_name}/fitting/{target_date}/
    """
    
    fs=20

    expt_list   = [60, 120, 240, 480]
    marker_list = ['o', 'X', '^', 's']
    ls_list     = ['-', '--', '-.', ':']
    colors_list = ['C0', 'C1', 'C2', 'C3']

    fig1 = plt.figure(figsize=(10,6), dpi=200, tight_layout=True)
    fig1.suptitle(target_name+" with "+filtername+": compare different parameters\n"+\
                  "Mid_Time="+Time(real_mid_time, format='jd').iso ,fontsize=fs)
    fig1.patch.set_facecolor('white')


    # plot !!!
    ax = fig1.add_subplot(1, 1, 1)
    ax.set_title(f"\"Mid_Time\" ({mode})",fontsize=fs*0.8)

    ax.plot([1,2,3,4,5,6,7],
        np.full(7, 0.0),
        color='k')     

    yt_max = 0
    yt_min = 0

    # ==========================================================================================================
    for exp_t in expt_list:
        for filtername in ["Twinkle_ch0_filter"]:
            df_exp   = df[df["exp_t [sec]"]==exp_t]

            df_filter_bool = df_exp["filtername"]==filtername

            tmid_obs      = df_exp[df_filter_bool]["$\Delta$tmid [min]"].values
            tmid_low_obs  = df_exp[df_filter_bool]["tmid_low [min]"].values
            tmid_high_obs = df_exp[df_filter_bool]["tmid_high [min]"].values

            yt_max = max(yt_max, max(tmid_obs + tmid_high_obs))
            yt_min = min(yt_min, min(tmid_obs + tmid_low_obs))

            # 檢查這一個binsize缺少了哪一些 要挑哪一些
            x_list = []
            bin_list_df   = df_exp[df_filter_bool]["binsize [min]"].values
            for bin_t_thisbinlist in bin_list_df:
                x_index = binsize_list.index(bin_t_thisbinlist)
                x_list.append(x_index)
            x_list = np.array(x_list)
    # ==========================================================================================================

        ax.errorbar(x_list+1, tmid_obs,
                    yerr = [-tmid_low_obs, tmid_high_obs],
                    fmt = '', markersize=8, capsize=8, lw=2,
                    color=colors_list[expt_list.index(exp_t)],
                    marker = marker_list[expt_list.index(exp_t)],
                    ls = ls_list[expt_list.index(exp_t)],
                    zorder = 7-expt_list.index(exp_t), alpha=1,
                    label = f"exp_t={exp_t}[sec]")

        ax.set_xlabel("Data binning [min]",fontsize=fs*0.6)
        ax.set_ylabel("$\Delta$Mid_T [min]",fontsize=fs*0.6)
        ax.tick_params(axis='y', labelsize= fs*0.5)

        xticks = ["non-binned", "bin5", "bin10", "bin15", "bin20", "bin25", "bin30"]
        # xticks = df[df['exp_t [sec]']==60][df[df['exp_t [sec]']==60]['filtername']==filtername]['binsize [min]'].values
        ax.set_xticks(np.arange(7)+1, labels=xticks, fontsize=fs*0.6)
        ax.axis(ymin=yt_min*1.5,ymax=yt_max*1.2)
        ax.legend(loc = 'lower right', prop={'size': fs*0.5}, ncol=4)

    # check if there is the save folder
    if not os.path.isdir(f"Result/{target_name}/fitting/{target_date}"):
        print(f"create the folder under path:Result/{target_name}/fitting/{target_date}")
        os.makedirs(f"Result/{target_name}/fitting/{target_date}")

    if save:
        plt.savefig(f"{filename}.png")
    plt.show()
    
    return





import os, copy, math, json
import numpy as np
import pandas as pd
from pylightcurve_master import pylightcurve_self as plc
# import pylightcurve as plc
import matplotlib.pyplot as plt
from astropy.time import Time


# This function is copied from pylight curve siurce code
# The main usage od this function is to generated the fp_over_fs of target planet,
# and can use this result to add new filter information

def self_fp_over_fs(rp_over_rs, sma_over_rs, albedo, emissivity, stellar_temperature, filter_name, filter_wave):
    # here, the self_fp_over_fs need the filter name to load the "transmission rate"
    
    def _black_body(w, t):
        # w in mu
        w = w / (10 ** 6)
        h = 6.62607004 * (10 ** (-34))
        c = 3 * (10 ** 8)
        w5 = w ** 5
        k = 1.38064852 * (10 ** (-23))
        return (2 * h * c * c / w5) / (np.exp(h * c / w / k / t) - 1)

    planet_temperature = stellar_temperature * np.sqrt(0.5 / sma_over_rs) * (((1 - albedo) / emissivity) ** 0.25)

    if filter_wave=='':
        wavelength_array, band = np.loadtxt(os.path.join("Data_tk/passband", f"{filter_name}.pass"), unpack=True)
    
    else:
        # getting the lower and upper bound of each channel
        low_wave  = int(float(filter_wave.split("_")[0]))
        high_wave = int(float(filter_wave.split("_")[1]))

        wavelength_array, band = np.loadtxt(os.path.join("Data_tk/passband", f"{filter_name}.pass"), unpack=True)

        # using the lower and upper bound to get the range information
        wavelength_array_filter = copy.copy(wavelength_array)
        wavelength_array = wavelength_array[wavelength_array>=low_wave]
        band             = band[wavelength_array_filter>=low_wave]
        wavelength_array_filter = copy.copy(wavelength_array)
        wavelength_array = wavelength_array[wavelength_array<=high_wave]
        band             = band[wavelength_array_filter<=high_wave]
    
    binsedge = 0.5 * (wavelength_array[:-1] + wavelength_array[1:])
    binsedge1 = np.append(wavelength_array[0] - (binsedge[0] - wavelength_array[0]), binsedge)
    binsedge2 = np.append(binsedge, wavelength_array[-1] + (wavelength_array[-1] - binsedge[-1]))
    binswidth = binsedge2 - binsedge1

    weights = band * binswidth / 10000
    emission = ((rp_over_rs ** 2) *
                _black_body(wavelength_array / 10000, planet_temperature) /
                _black_body(wavelength_array / 10000, stellar_temperature))

    emission = np.sum(emission * weights) / np.sum(weights)

    reflection = albedo * (rp_over_rs ** 2) / (sma_over_rs ** 2)

    return reflection + emission


def get_noise(planet, filtername, exp_t, time, observable, filter_channelornot, filterchannelnum, verbose):
    """
    <<input>>
    planet              : pylightcurve planet object ; the inforamtion of planet parameter       ; generated from func:create_planet_tkdata
    filtername          : string                     ; filtername by given to get the error information
    exp_t               : float                      ; the exposure time
    time                : np.array(float)            ; the time array "without" considering Earth block 
    observable          : np.array(float)            ; the time array "with" considering Earth block
    filter_channelornot : boolen                     ; the current filter is channeled or not
    filterchannelnum    : int                        ; the number of channeling filter if the current filter is channeled
    verbose             : boolen                     ; print details or not
    
    <<output>>
    scatter_mod     : np.array(float)            ; the flux array points (not yet add noise) "without" considering Earth block
    scatter_obs     : np.array(float)            ; the flux array points (not yet add noise) "with" considering Earth block
    err_per_exp     : float                      ; the error(noise) of each exposure time    ; unit:[ppm]
    """
    
    # Loading the information from .csv data base
    stellar_name    = '-'.join(planet.name.split('-')[:-1])
    errcsv_CHEOPS   = pd.read_csv(f"Data_tk/error/{stellar_name}/{stellar_name}_CHEOPS-error.csv")
    errcsv_Twinkle  = pd.read_csv(f"Data_tk/error/{stellar_name}/{stellar_name}_Twinkle-error.csv") 
    
    # CHEOPS will use the url: Exposure Time Calculator to get the error information
    #############################################################################
    if filtername.split("_")[0] == "CHEOPS":
        if filter_channelornot:
            print("CHEOPS can not apply error for channeling filter now.")
            print("Beacuse we don't have CHEOPS error source of channeling filter.")
            print("You can use the not add noise option!!!")
            print("")
            return np.zeros(len(time)), np.zeros(len(observable)), 0.
        
        expt_column  = pd.Index(errcsv_CHEOPS["exposure time(s)"])       # get the column of exposure time(s)
        expt_index   = expt_column.get_loc(exp_t)                        # get the index of target exp_t
        err_per_exp  = errcsv_CHEOPS.loc[expt_index,"photon noise[ppm]"]

        if verbose:
            print(f"The exp_t for CHEOPS filter is {exp_t} [sec]")
            print(f"The err_per_exp is {err_per_exp} [ppm]")
            print("")
    #############################################################################

    # Twinkle will use the url: Stardrive-Radiometric tool to get the error information
    #############################################################################
    elif filtername.split("_")[0] == "Twinkle":
        if filter_channelornot and filterchannelnum!="default":
            print("Twinkle can not apply error for self-divided channeling filter now.")
            print("You can use the not add noise option!!!")
            print("")
            return np.zeros(len(time)), np.zeros(len(observable)), 0.
        
        filter_column  = pd.Index(errcsv_Twinkle["filtername"])           # get the column of error_per_exposure
        filter_index   = filter_column.get_loc(filtername)                # get the index (which filter)
        err_per_exp_tk = errcsv_Twinkle.loc[filter_index, "error_per_exposure"]

        # Beacause the Stardrive-Radiometric tool can not modify the exposure time by user
        # (exposure time was calculated by tool and give an appropriate exposure time.)
        # We will check if the user enter the same exposure time as tool gave us,
        # or recalculate the error by the relation of error proportional to 1/(exp_t)^0.5
        if exp_t == errcsv_Twinkle.loc[filter_index, "exposure_time"]:
            err_per_exp = err_per_exp_tk
            if verbose:
                print(f"The self-define exp_t {exp_t} [sec] for {filtername} is the same as Twinkle tool given")
                print(f"The err_per_exp is {err_per_exp} [ppm]")
                print("")

        else:
            exp_t_tk = errcsv_Twinkle.loc[filter_index, "exposure_time"]
            err_per_exp = round(err_per_exp_tk*math.sqrt(exp_t_tk/exp_t), 3)
            if verbose:
                print(f"The self-define exp_t {exp_t} [sec] for {filtername} is the different from Twinkle tool given {exp_t_tk} [sec]")
                print(f"The Twinkle given err_per_exp is {err_per_exp_tk} [ppm]")
                print(f"The self-define err_per_exp after transform is {err_per_exp} [ppm]")
                print("")
    #############################################################################

    scatter_mod = np.random.normal(0,err_per_exp/1e6,len(time))
    scatter_obs = np.random.normal(0,err_per_exp/1e6,len(observable))
    
    return scatter_mod, scatter_obs, err_per_exp



def model_transit(planet, exposure_list,
                  target_lc_num, tk_avail_time,
                  filtername_list,
                  filterchannelnum_list,
                  filtername_allchannellist,
                  add_noise=True,
                  verbose=True):
    """
    <<input>>
    planet                    : pylightcurve planet object ; the inforamtion of planet parameter                   ; generated from func:create_planet_tkdata
    exposure_list             : list(int)                  ; different exposure time list 
    target_lc_num             : int                        ; the index of target_date in tk_avail_time
    tk_avail_time             : dictionary                 ; available observation infromation                     ; generated from Twinkle Stardrive orbit tool
    filtername_list           : list(str)                  ; different filter name 
    filterchannelnum_list     : list(int)                  ; the number of channels in each filter                 ; determined in LC_syn_tk.ipynb
    filtername_allchannellist : list(string)               ; the name list of all including origin/channel filters ; generated from LC_syn_tk.ipynb
    add_noise                 : boolen                     ; whether adding noise to model light curve or not
    verbose                   : boolen                     ; print details or not
    
    <<output>>
    data            : dictionary                 ; synthetic light curve data                ; generated from LC_syn_tk.py
    """
    
    # create the dictionary to save the syntheic light curve
    mid_time    = planet.mid_time
    target_date = Time(mid_time, format='jd')
    target_date = Time(target_date.iso, out_subfmt='date').iso
    start_time  = Time(tk_avail_time["transit"][target_lc_num]["start_time"]).jd

    data = {}
    data["mid_time"]            = mid_time
    data["target_date"]         = target_date
    data["exposure_list [sec]"] = exposure_list
    data["filtername_list"]     = filtername_list
    data["planet_object"]       = planet
    data["time_array"]          = {}
    data["observable_array"]    = {}
    data["orbit_array"]         = {}
    data["err_per_exp [ppm]"]   = {}
    
    data["tr_mod_array"]        = {}
    data["tr_obs_array"]        = {}
    data["mod_noise_array"]     = {}
    data["obs_noise_array"]     = {}
    
    
    filter_index_check = -1
    for filtername in filtername_allchannellist:
        # check which filter is the channeling filter belong.
        #############################################################################
        filter_channel_now = filtername.split('-')[0]                   # the belong's original filter of the channeled filter
        filter_index       = filtername_list.index(filter_channel_now)  # the index of this belong's original filter
        
        # also check if filter have channel or not (determine by check the white filter)
        if len(filtername.split('-'))==1:
            if not filterchannelnum_list[filter_index]==0:
                filter_channelornot = True 
                print(f"Check for {filtername} channeling: True")  

            else:
                filter_channelornot = False
        
        # check if this channeling filter is the same as the belong's original filter
        # case1(>): this channeling filter is belong to belong's original filter (In general, will be the )     
        # case2(==): this channeling filter is belong to next filter, update belong's original filter
        if filter_index>=filter_index_check: 
            filter_index_check = filter_index
        
        # case3(<): belong's original filter is the next filter,
        #           so channeling filter should skip those filters until round to the next belong's filter
        else:
            continue
        #############################################################################
        
        data["time_array"][filtername]        = {}
        data["observable_array"][filtername]  = {}
        data["orbit_array"][filtername]       = {}
        data["err_per_exp [ppm]"][filtername] = {}
        
        data["tr_mod_array"][filtername]      = {}
        data["tr_obs_array"][filtername]      = {}
        data["mod_noise_array"][filtername]   = {}
        data["obs_noise_array"][filtername]   = {}
        
        # the channeling filters will use all the same exposure time in the exposure_list
        for exp_t in exposure_list[filter_index]:        
            # create a time array that is based on the exposure time
            masknum = int((mid_time - start_time)*24*60*60/exp_t)

            time    = np.linspace(mid_time-masknum*exp_t/60/60/24,
                                  mid_time+masknum*exp_t/60/60/24,
                                  2*masknum+1)
            
            # create a time array that can do observation (consider shading from earth)
            #############################################################################
            observable = []
            num_orbits = tk_avail_time["transit"][target_lc_num]["num_views"]
            data["orbit_array"][filtername][f"exp_t{exp_t} [sec]"] = {}
            
            for ii in range(int(num_orbits)):
                orbit_start    = Time(tk_avail_time["transit"][target_lc_num]["viewing_times"][ii][0]).jd
                orbit_end      = Time(tk_avail_time["transit"][target_lc_num]["viewing_times"][ii][1]).jd

                filterdata = list(filter(lambda x: x>=orbit_start and x<=orbit_end, time))
                observable = np.append(observable, filterdata)
                
                data["orbit_array"][filtername][f"exp_t{exp_t} [sec]"][f"orbit_{ii}"] = np.array(filterdata)
            #############################################################################
                
            data["time_array"][filtername][f"exp_t{exp_t} [sec]"]        = time
            data["observable_array"][filtername][f"exp_t{exp_t} [sec]"]  = observable
                 
            # create the error array based on the information from the telescope .csv file
            #############################################################################
            if add_noise:            
                scatter_mod, scatter_obs, err_per_exp = get_noise(planet, filtername, exp_t, time, observable,
                                                                  filter_channelornot, filterchannelnum_list[filter_index], verbose)
                data["err_per_exp [ppm]"][filtername][f"exp_t{exp_t} [sec]"] = err_per_exp
                
                if scatter_mod.all()==0 or scatter_obs.all()==0 or err_per_exp==0.:
                    # belong's original filter is the next filter,
                    # so channeling filter should skip those filters until round to the next belong's filter
                    filter_index_check = filter_index_check + 1
                    break
            #############################################################################


            # the model of whole transit with "no shaded" by earth
            tr_mod = planet.transit_integrated(time, time_format="BJD_TDB", 
                                               exp_time=exp_t, time_stamp = "mid", 
                                               filter_name = filtername, 
                                               max_sub_exp_time=1)

            # the model of whole transit with "shaded" by earth
            tr_obs = planet.transit_integrated(observable, time_format="BJD_TDB", 
                                               exp_time=exp_t, time_stamp = "mid", 
                                               filter_name = filtername, 
                                               max_sub_exp_time=1)

            data["tr_mod_array"][filtername][f"exp_t{exp_t} [sec]"]      = tr_mod
            data["tr_obs_array"][filtername][f"exp_t{exp_t} [sec]"]      = tr_obs

            if add_noise:
                data["mod_noise_array"][filtername][f"exp_t{exp_t} [sec]"]   = tr_mod+scatter_mod
                data["obs_noise_array"][filtername][f"exp_t{exp_t} [sec]"]   = tr_obs+scatter_obs
                
    return data


def create_planet_tkdata(tk_datacat, tk_avail_time, target_lc_num,
                         ldc_stellar_model = "phoenix"):
    """
    <<input>>
    tk_datacat       : dictionary ; the information of planet on Twinkle catalogue      ; generated from Twinkle Stardrive database
    tk_avail_time    : dictionary ; available observation infromation                   ; generated from Twinkle Stardrive tool(orbit)
    target_lc_num    : int        ; the index of target_date in tk_avail_time
    ldc_stellar_model: string     ; the model to generate the stellar intensity profile ; default = phoenix
    
    <<output>>
    planet        : pylightcurve planet object ; synthetic light curve data     ; generated from func:create_planet_tkdata
    """
    
    # save useful variable from "Stardrive orbital tool"  
    start_time = Time(tk_avail_time["transit"][target_lc_num]["start_time"]).jd
    end_time   = Time(tk_avail_time["transit"][target_lc_num]["end_time"]).jd
    mid_time   = Time((start_time+end_time)/2., format='jd').jd 

    
    # create a plc.planet object by using "Stardrive database"
    planet = plc.Planet(name = tk_datacat["planet_name"].split(" ")[0]+"-"+tk_datacat["planet_name"].split(" ")[1], 

                ra = plc.Degrees(str(tk_datacat["star_ra"])),     # float values are assumed to be in degrees,
                                                                  # alternatively, you can provide a plc.Hours or plc.Degrees object
                                                                  # here it would be plc.Hours('22:03:10.7729')

                dec =  plc.Degrees(str(tk_datacat["star_dec"])),  # float values are assumed to be in degrees,
                                                                  # alternatively, you can provide a plc.Hours or plc.Degrees object
                                                                  # here it would be plc.Degrees('+18:53:03.548')

                stellar_logg = tk_datacat["star_logg"],               # float, in log(cm/s^2) 
                stellar_temperature = tk_datacat["star_temperature"], # float, in Kelvin
                stellar_metallicity = tk_datacat["star_metallicity"], # float, in dex(Fe/H) or dex(M/H)
                rp_over_rs = tk_datacat["rp_over_rs"],                # float, no units
                period = tk_datacat["planet_period"],                 # float, in days
                sma_over_rs = tk_datacat["sma_over_rs"],              # float, no units
                eccentricity = tk_datacat["eccentricity"],            # float, no units
                inclination = tk_datacat["inclination"],              # float values are assumed to be in degrees,
                                                                      # alternatively, you can provide a plc.Hours or plc.Degrees object
                                                                      # here it would be plc.Degrees(86.71)           

                periastron = tk_datacat["periastron"],                # float values are assumed to be in degrees,
                                                                      # alternatively, you can provide a plc.Hours or plc.Degrees object
                                                                      # here it would be plc.Degrees(0.0)

                mid_time = mid_time,                                  # float, in days
                mid_time_format = "BJD_TDB",                          # str, available formats are 
                                                                      #      JD_UTC, MJD_UTC, HJD_UTC, HJD_TDB, BJD_UTC, BJD_TDB    
                ldc_method = "claret",                                # str, default = claret, the other methods are: linear, quad, sqrt
                ldc_stellar_model = ldc_stellar_model,                # str, default = phoenix, the other model is atlas
                albedo = tk_datacat["planet_albedo"],                 # float, default = 0.15, no units 
                emissivity = 1.0                                      # float, default = 1.0, no units
                ) 
    return planet



import pandas as pd
import json
from astropy.time import Time
import cartopy.crs as ccrs
import matplotlib.pyplot as plt
import numpy as np
from scipy.interpolate import interp1d
from matplotlib.colors import BoundaryNorm
from matplotlib.ticker import MaxNLocator
from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER



def plot_avail(data):
    
    fs=18
    plt.figure(figsize=(10,6))

    for ii in range(np.shape(data)[0]):
    
        times = data_avail[ii]
        t = Time(times, format='isot', scale='utc')
        year = int(data[ii][0][0:4])
    
        year_start = Time([f'{year}-01-01T00:00:00.000'], format='isot', scale='utc')
        day_start = t[0].jd - year_start.jd
        day_end = t[1].jd - year_start.jd
    
        plt.plot([day_start,day_end],[year,year],lw=4,c='r')
        plt.scatter([day_start,day_end],[year,year],c='r',marker='d',s=80)

    plt.xlim(0,365)
    plt.ylim(2023,2030)
    plt.xlabel('Day',fontsize=fs)
    plt.ylabel('Year',fontsize=fs)
    plt.xticks(fontsize=fs-2)
    plt.yticks(fontsize=fs-2)
    plt.show()
    
    
def plot_cartopy(data, num_levels):
    
    X = data['full_ra']
    Y = data['full_dec']
    plot_data = data['full_coverage']
    
    #sun = pd.read_csv('sun_coord.csv')
    #interp_ecliptic = interp1d(sun['RA'],sun['Dec'],bounds_error=None, fill_value='extrapolate')

   #ecliptic_dec = interp_ecliptic(X)
    
    fs = 14
    
    levels = MaxNLocator(nbins=num_levels).tick_values(np.min(plot_data)-1, np.max(plot_data)+1)
    cmap = plt.get_cmap('Spectral_r')
    norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)
    
    #plt.style.use("dark_background")
    
    fig = plt.figure(figsize=(10,6))
    
    ax = fig.add_subplot(1, 1, 1, projection=ccrs.Mollweide(central_longitude=180))
    
    cs = ax.contourf(X,Y,plot_data, levels = levels,
                transform=ccrs.PlateCarree(),
                cmap=cmap, norm = norm)
    
    cbar = plt.colorbar(cs, ax = ax, shrink = 0.6)
    cbar.ax.set_ylabel('Total Time Available [Days/Year]', fontsize = fs-2)
    
    ax.scatter(data['ra'],data['dec'],c='w',edgecolor='k',transform=ccrs.PlateCarree(),lw=0.5)
      
    gl = ax.gridlines(crs=ccrs.PlateCarree(), draw_labels=True,
                  linewidth=0.5, color='black', linestyle=':')
    gl.bottom_labels = False
    gl.top_labels = False
    gl.left_labels = False
    gl.right_labels = False
    gl.xlines = True
    gl.ylines = True
    
    text_ra = [60,120,180,240,300]
    for ii in text_ra:
        if ii < 100:
            ax.text(ii-5,-2.5,f'{ii}$^\circ$',transform=ccrs.PlateCarree(),fontsize=fs)
        else:
            ax.text(ii-10,-2.5,f'{ii}$^\circ$',transform=ccrs.PlateCarree(),fontsize=fs)
            
    gl.xformatter = LONGITUDE_FORMATTER
    gl.yformatter = LATITUDE_FORMATTER
    
    ax.text(-0.07, 0.55, 'Declination', va='bottom', ha='center',
        rotation='vertical', rotation_mode='anchor',
        transform=ax.transAxes, fontsize = fs)
        
    ax.text(0.5, -0.2, 'Right Ascension', va='bottom', ha='center',
        rotation='horizontal', rotation_mode='anchor',
        transform=ax.transAxes, fontsize = fs)
    
    ax.text(-0.02, 0.475, '0$^\circ$', va='bottom', ha='center',
        rotation='horizontal', rotation_mode='anchor',
        transform=ax.transAxes, fontsize = fs)
    ax.text(0.015, 0.68, '30$^\circ$', va='bottom', ha='center',
        rotation='horizontal', rotation_mode='anchor',
        transform=ax.transAxes, fontsize = fs)
    ax.text(0.14, 0.87, '60$^\circ$', va='bottom', ha='center',
        rotation='horizontal', rotation_mode='anchor',
        transform=ax.transAxes, fontsize = fs)
    ax.text(0.015, 0.24, '-30$^\circ$', va='bottom', ha='center',
        rotation='horizontal', rotation_mode='anchor',
        transform=ax.transAxes, fontsize = fs)
    ax.text(0.14, 0.06, '-60$^\circ$', va='bottom', ha='center',
        rotation='horizontal', rotation_mode='anchor',
        transform=ax.transAxes, fontsize = fs)

    plt.show()
    
    

def plot_transit(data,lc_num):
    
    t_start  = Time(data['transit'][lc_num]['start_time'])
    t_end    = Time(data['transit'][lc_num]['end_time'])
    
    t_mid    = Time((t_start.jd + t_end.jd)/2., format='jd')
    mid_date = Time(t_mid.iso , out_subfmt='date').iso

    time     = np.linspace(t_start.jd,t_end.jd,len(data['lc']))

    fs = 18
    plt.figure(figsize=(8,4))
    plt.plot(time,data['lc'],c='k',lw=2)

    for ii in range(len(data['transit'][lc_num]['viewing_times'])):
        view_start = Time(data['transit'][lc_num]['viewing_times'][ii][0])
        view_end = Time(data['transit'][lc_num]['viewing_times'][ii][1])
        plt.fill_between([view_start.jd,view_end.jd],[0.1,0.1],[1.5,1.5],color='g',alpha=0.2, label=f'Visable Channel{ii+1}')

    td = 1-min(data['lc'])
    plt.xlim(min(time),max(time))
    plt.ylim(1-1.4*td,1. + td*0.2)
    plt.xlabel('Time [BJD]',fontsize=fs)
    plt.ylabel('Relative Flux',fontsize=fs)
    plt.xticks(fontsize=fs-2)
    plt.yticks(fontsize=fs-2)
    plt.legend()
    plt.title(f"Date: {mid_date} | Coverage: {np.round(data['transit'][lc_num]['coverage']*100,1)}%",fontsize=fs-2)
    plt.show()
    

def replot_star_data(data, col):
    
    fs=18
    plt.figure(figsize=(10,6))
    plt.scatter(data['ch0']['wavelength'],data['ch0'][col],c='w',edgecolor='b')
    plt.scatter(data['ch1']['wavelength'],data['ch1'][col],c='w',edgecolor='r')
    plt.xlim(0.5,4.5)
    plt.xlabel('Wavelength [$\mu$m]',fontsize=fs)
    if col == 'error_per_hour':
        plt.ylabel('Error Per Hour [ppm]',fontsize=fs)
    elif col == 'error_per_exposure':
        plt.ylabel('Error Per Exposure [ppm]',fontsize=fs)
        plt.title(f"Exposure Time: {np.round(data['exposure_time'],2)} s",fontsize=fs-4)
    plt.xticks(fontsize=fs-2)
    plt.yticks(fontsize=fs-2)
    plt.show()
    
    
def replot_transit_data(data, num_sh):
    
    fs=18
    plt.figure(figsize=(10,6))
    plt.scatter(data['ch0']['wavelength'],data['ch0'][f'transit_snr_{num_sh}sh'],c='w',edgecolor='b')
    plt.scatter(data['ch1']['wavelength'],data['ch1'][f'transit_snr_{num_sh}sh'],c='w',edgecolor='r')
    plt.xlim(0.5,4.5)
    plt.xlabel('Wavelength [$\mu$m]',fontsize=fs)
    plt.ylabel(f'Transit SNR ({num_sh} Scale Heights)',fontsize=fs)
    plt.xticks(fontsize=fs-2)
    plt.yticks(fontsize=fs-2)
    plt.show()
    
    fs=18
    plt.figure(figsize=(10,6))
    plt.scatter(data['ch0']['wavelength'],data['ch0']['eclipse_snr'],c='w',edgecolor='b')
    plt.scatter(data['ch1']['wavelength'],data['ch1']['eclipse_snr'],c='w',edgecolor='r')
    plt.xlim(0.5,4.5)
    plt.xlabel('Wavelength [$\mu$m]',fontsize=fs)
    plt.ylabel('Eclipse SNR',fontsize=fs)
    plt.xticks(fontsize=fs-2)
    plt.yticks(fontsize=fs-2)
    plt.show()